from insights.core.context import OSP
from insights.contrib.pyparsing import ParseException as PyparsingParseException
from insights.parsers.rabbitmq import RabbitMQReport, RabbitMQReportOfContainers
from insights.tests import context_wrap

osp_controller = OSP()
osp_controller.role = "Controller"


RABBITMQCTL_REPORT_0 = """
Error: unable to connect to node rabbit@dprcclab002: nodedown

DIAGNOSTICS
===========

attempted to contact: [rabbit@dprcclab002]

rabbit@dprcclab002:
  * connected to epmd (port 4369) on dprcclab002
  * epmd reports: node 'rabbit' not running at all
                  no other nodes on dprcclab002
  * suggestion: start the node

current node details:
- node name: 'rabbitmq-cli-08@dprcclab002'
- home dir: /var/lib/rabbitmq
- cookie hash: k0XaAkNZ29GQG22RA/Uyxw==
"""

RABBITMQCTL_REPORT_1 = """
Reporting server status on {{2016,10,17},{3,42,41}}

 ...
Status of node rabbit@rabbitmq0 ...
[{pid,5085},
 {os,{unix,linux}},
 {erlang_version,
     "Erlang R16B03-1 (erts-5.10.4) [source] [64-bit] [smp:4:4] [async-threads:30] [hipe] [kernel-poll:true]\n"},
 {listeners,[{clustering,25672,"::"},{amqp,5672,"::"}]},
 {vm_memory_high_watermark,0.4},
 {vm_memory_limit,1590132736},
 {disk_free_limit,50000000},
 {disk_free,40712921088},
 {file_descriptors,
     [{total_limit,924},{total_used,3},{sockets_limit,829},{sockets_used,1}]},
 {processes,[{limit,1048576},{used,180}]},
 {run_queue,0},
 {uptime,5590}]

Cluster status of node rabbit@rabbitmq0 ...
[{nodes,[{disc,[rabbit@rabbitmq0]}]},

Application environment of node rabbit@rabbitmq0 ...
[{auth_backends,[rabbit_auth_backend_internal]},

Permissions on /:
user    configure       write   read
guest   .*      .*      .*
redhat  redhat.*        .*      .*
redhat1 redhat.*        .*      .*

Permissions on test_vhost:

Policies on /:

Policies on test_vhost:

...done.
"""

RABBITMQCTL_REPORT_2 = """
Reporting server status on {{2016,4,26},{6,51,27}}

 ...
Status of node 'rabbit@overcloud-controller-2' ...
[{pid,4087},
 {vm_memory_limit,26943271731},
 {disk_free_limit,50000000},
 {disk_free,257304219648},
 {file_descriptors,[{total_limit,3996},
                    {total_used,835},
                    {sockets_limit,3594},
                    {sockets_used,833}]},
 {uptime,3075474}]

Cluster status of node 'rabbit@overcloud-controller-2' ...
[{nodes,[{disc,['rabbit@overcloud-controller-0',
                'rabbit@overcloud-controller-1',
                'rabbit@overcloud-controller-2']}]},
 {partitions,[]}]

Application environment of node 'rabbit@overcloud-controller-2' ...
[{auth_backends,[rabbit_auth_backend_internal]}]

Status of node 'rabbit@overcloud-controller-1' ...
[{pid,9304},
 {disk_free_limit,50000000},
 {disk_free,260561866752},
 {file_descriptors,[{total_limit,3996},
                    {total_used,853},
                    {sockets_limit,3594},
                    {sockets_used,851}]},
 {uptime,3075482}]

Cluster status of node 'rabbit@overcloud-controller-2' ...
[{nodes,[{disc,['rabbit@overcloud-controller-0',
                'rabbit@overcloud-controller-1',
                'rabbit@overcloud-controller-2']}]},
 {partitions,[]}]

Application environment of node 'rabbit@overcloud-controller-2' ...
[{auth_backends,[rabbit_auth_backend_internal]}]

Status of node 'rabbit@overcloud-controller-0' ...
[{pid,6005},
 {disk_free_limit,50000000},
 {disk_free,259739344896},
 {file_descriptors,[{total_limit,3996},
                    {total_used,967},
                    {sockets_limit,3594},
                    {sockets_used,965}]},
 {uptime,3075485}]

Cluster status of node 'rabbit@overcloud-controller-2' ...
[{nodes,[{disc,['rabbit@overcloud-controller-0',
                'rabbit@overcloud-controller-1',
                'rabbit@overcloud-controller-2']}]},
 {partitions,[]}]

Application environment of node 'rabbit@overcloud-controller-2' ...
[{auth_backends,[rabbit_auth_backend_internal]}]

Permissions on /:
user	configure	write	read
guest	.*	.*	.*

Policies on /:
vhost	name	apply-to	pattern	definition	priority
/	ha-all	all	^(?!amq\\.).*	{"ha-mode":"all"}	0

Parameters on /:

...done.
"""

RABBITMQCTL_REPORT_3 = """
Reporting server status on {{2019,5,2},{18,57,48}}

 ...
Status of node rabbit@controller1 ...
[{pid,76101},
 {running_applications,[{rabbit,"RabbitMQ","3.6.3"},
                        {os_mon,"CPO  CXC 138 46","2.4"},
                        {rabbit_common,[],"3.6.3"},
                        {mnesia,"MNESIA  CXC 138 12","4.13.4"},
                        {ranch,"Socket acceptor pool for TCP protocols.",
                               "1.2.1"},
                        {xmerl,"XML parser","1.3.10"},
                        {sasl,"SASL  CXC 138 11","2.7"},
                        {stdlib,"ERTS  CXC 138 10","2.8"},
                        {kernel,"ERTS  CXC 138 10","4.2"}]},
 {os,{unix,linux}},
 {erlang_version,"Erlang/OTP 18 [erts-7.3.1.2] [source] [64-bit] [smp:56:56] [async-threads:896] [hipe] [kernel-poll:true]\x5c\x6e"},
 {memory,[{total,7198193032},
          {connection_readers,2582888},
          {connection_writers,611032},
          {connection_channels,3622776},
          {connection_other,6805920},
          {queue_procs,113304336},
          {queue_slave_procs,2454511224},
          {plugins,0},
          {other_proc,59823544},
          {mnesia,18861888},
          {mgmt_db,0},
          {msg_index,2912704},
          {other_ets,3109632},
          {binary,4460926336},
          {code,19689791},
          {atom,752537},
          {other_system,50678424}]},
 {alarms,[]},
 {listeners,[{clustering,25672,"::"},{amqp,5672,"172.16.64.62"}]},
 {vm_memory_high_watermark,0.4},
 {vm_memory_limit,162189606912},
 {disk_free_limit,50000000},
 {disk_free,151495786496},
 {file_descriptors,[{total_limit,16284},
                    {total_used,222},
                    {sockets_limit,14653},
                    {sockets_used,220}]},
 {processes,[{limit,1048576},{used,16029}]},
 {run_queue,0},
 {uptime,1986766},
 {kernel,{net_ticktime,60}}]

Cluster status of node rabbit@controller1 ...
[{nodes,[{disc,[rabbit@controller0,rabbit@controller1,rabbit@controller2]}]},
 {running_nodes,[rabbit@controller0,rabbit@controller2,rabbit@controller1]},
 {cluster_name,<<"rabbit@controller0.external.s4-southlake.vcp.vzwops.com">>},
 {partitions,[]},
 {alarms,[{rabbit@controller0,[]},
          {rabbit@controller2,[]},
          {rabbit@controller1,[]}]}]

Permissions on /:
user	configure	write	read
guest	.*	.*	.*

Policies on /:
vhost	name	apply-to	pattern	definition	priority
/	ha-all	all	^(?!amq\\.).*	{"ha-mode":"all"}	0

Parameters on /:

...done.

"""


def test_rabbitmq_report():
    result = RabbitMQReport(context_wrap(RABBITMQCTL_REPORT_1,
            hostname="controller_1", osp=osp_controller)).result
    assert result.get("nstat").get("rabbit@rabbitmq0").\
            get("file_descriptors").get("total_limit") == "924"
    permissions = {'/': {'redhat1': ['redhat.*', '.*', '.*'],
                        'guest': ['.*', '.*', '.*'],
                        'redhat': ['redhat.*', '.*', '.*']},
                   'test_vhost': ''}
    assert result.get("perm") == permissions

    result = RabbitMQReport(context_wrap(RABBITMQCTL_REPORT_2,
            hostname="controller_1", osp=osp_controller)).result
    assert result.get("nstat").get("'rabbit@overcloud-controller-2'").\
            get("file_descriptors").get("total_limit") == '3996'
    assert len(result.get("nstat")) == 3
    permissions = {'/': {'guest': ['.*', '.*', '.*']}}
    assert result.get("perm") == permissions

    result = RabbitMQReport(context_wrap(RABBITMQCTL_REPORT_3,
            hostname="controller_1", osp=osp_controller)).result
    assert result.get("nstat").get("rabbit@controller1").\
            get("file_descriptors").get("total_limit") == '16284'
    assert len(result.get("nstat")) == 1
    assert result.get("nstat").get("rabbit@controller1").\
            get("erlang_version")[-1] == '[kernel-poll:true]\x5cn"'


def test_rabbitmq_report_with_parse_exception():
    try:
        RabbitMQReport(context_wrap(RABBITMQCTL_REPORT_0,
            hostname="controller_1", osp=osp_controller))
        assert False
    except PyparsingParseException:
        assert True


def test_rabbitmq_report_of_containers():
    result = RabbitMQReportOfContainers(context_wrap(RABBITMQCTL_REPORT_1,
            hostname="controller_1", osp=osp_controller)).result
    assert result.get("nstat").get("rabbit@rabbitmq0").\
            get("file_descriptors").get("total_limit") == "924"
    permissions = {'/': {'redhat1': ['redhat.*', '.*', '.*'],
                        'guest': ['.*', '.*', '.*'],
                        'redhat': ['redhat.*', '.*', '.*']},
                   'test_vhost': ''}
    assert result.get("perm") == permissions



"""
YumConf - file ``/etc/yum.conf``
================================
"""
import six

from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.parsr.iniparser import NoOptionError
from insights.specs import Specs


@parser(Specs.yum_conf)
class YumConf(IniConfigFile):
    """
    This module provides parsing for the ``/etc/yum.conf`` file.
    The ``YumConf`` class parses the information in the file
    ``/etc/yum.conf``. See the ``IniConfigFile`` class for more
    information on attributes and methods.

    Sample input data looks like::

        [main]
        cachedir=/var/cache/yum/$basearch/$releasever
        keepcache=0
        debuglevel=2
        logfile=/var/log/yum.log
        exactarch=1
        obsoletes=1
        gpgcheck=1
        plugins=1
        installonly_limit=3

        [rhel-7-server-rpms]
        metadata_expire = 86400
        baseurl = https://cdn.redhat.com/content/rhel/server/7/$basearch
        name = Red Hat Enterprise Linux 7 Server (RPMs)
        gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
        enabled = 1
        gpgcheck = 1

    Examples:
        >>> type(yconf)
        <class 'insights.parsers.yum_conf.YumConf'>
        >>> 'main' in yconf
        True
        >>> 'rhel-7-server-rpms' in yconf
        True
        >>> yconf.has_option('main', 'gpgcheck')
        True
        >>> yconf.has_option('main', 'foo')
        False
        >>> yconf.get('rhel-7-server-rpms', 'enabled')
        '1'
        >>> expected = {'obsoletes': '1', 'keepcache': '0', 'logfile': '/var/log/yum.log', 'cachedir': '/var/cache/yum/$basearch/$releasever', 'exactarch': '1', 'plugins': '1', 'gpgcheck': '1', 'debuglevel': '2', 'installonly_limit': '3'}
        >>> yconf.items('main') == expected
        True
    """
    def parse_content(self, content):
        super(YumConf, self).parse_content(content)
        # File /etc/yum.conf may contain repos definitions.
        # Keywords 'gpgkey' and 'baseurl' might contain multiple
        # values separated by comma. Convert those values into a list.
        for section in self.sections():
            for key in ('gpgkey', 'baseurl'):
                try:
                    value = self.get(section, key)
                    if value and isinstance(value, six.string_types):
                        self.set(section, key, value.split(','))
                except NoOptionError:
                    pass



import doctest

from insights.parsers import containers_policy
from insights.parsers.containers_policy import ContainersPolicy
from insights.tests.parsers import skip_exception_check
from insights.tests import context_wrap

CONTAINERS_POLICY_FILE = '''
{
  "default": [
    {
      "type": "insecureAcceptAnything"
    }
  ],
  "transports": {
    "docker": {
      "registry.access.redhat.com": [
        {
          "type": "signedBy",
          "keyType": "GPGKeys",
          "keyPath": "/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"
        }
      ],
      "registry.redhat.io/redhat/redhat-operator-index": [
        {
          "type": "insecureAcceptAnything"
        }
      ],
      "registry.redhat.io": [
        {
          "type": "signedBy",
          "keyType": "GPGKeys",
          "keyPath": "/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"
        }
      ]
    },
    "docker-daemon": {
      "": [
        {
          "type": "insecureAcceptAnything"
        }
      ]
    }
  }
}
'''.strip()


def test_doc_examples():
    env = {
        'containers_policy': ContainersPolicy(context_wrap(CONTAINERS_POLICY_FILE)),
    }
    failed, total = doctest.testmod(containers_policy, globs=env)
    assert failed == 0


def test_containers_policy():
    conf = ContainersPolicy(context_wrap(CONTAINERS_POLICY_FILE))
    assert len(conf["default"]) == 1
    assert conf["default"][0]["type"] == "insecureAcceptAnything"


def test_containers_policy_empty():
    assert 'Empty output.' in skip_exception_check(ContainersPolicy)



import doctest

from insights.parsers import ceilometer_conf
from insights.tests import context_wrap

CEILOMETER_CONTENT = """
[DEFAULT]
#
# From ceilometer
#
# To reduce polling agent load, samples are sent to the notification
# agent in a batch. To gain higher throughput at the cost of load set
# this to False. (boolean value)
#batch_polled_samples = true
# To reduce large requests at same time to Nova or other components
# from different compute agents, shuffle start time of polling task.
# (integer value)
#shuffle_time_before_polling_task = 0
# Configuration file for WSGI definition of API. (string value)
#api_paste_config = api_paste.ini
# Polling namespace(s) to be used while resource polling (unknown
# type)
#polling_namespaces = ['compute', 'central']
# List of pollsters (or wildcard templates) to be used while polling
# (unknown type)
#pollster_list = []
# Exchange name for Nova notifications. (string value)
#nova_control_exchange = nova
# List of metadata prefixes reserved for metering use. (list value)
#reserved_metadata_namespace = metering.
# Limit on length of reserved metadata values. (integer value)
#reserved_metadata_length = 256
# List of metadata keys reserved for metering use. And these keys are
# additional to the ones included in the namespace. (list value)
#reserved_metadata_keys =
# Inspector to use for inspecting the hypervisor layer. Known
# inspectors are libvirt, hyperv, vmware, xenapi and powervm. (string
# value)
#hypervisor_inspector = libvirt
# Libvirt domain type. (string value)
# Allowed values: kvm, lxc, qemu, uml, xen
#libvirt_type = kvm
# Override the default libvirt URI (which is dependent on
# libvirt_type). (string value)
#libvirt_uri =
# Dispatcher to process data. (multi valued)
# Deprecated group/name - [collector]/dispatcher
#dispatcher = database
# Number of items to request in each paginated Glance API request
# (parameter used by glancecelient). If this is less than or equal to
# 0, page size is not specified (default value in glanceclient is
# used). (integer value)
#glance_page_size = 0
# Exchange name for Ironic notifications. (string value)
#ironic_exchange = ironic
# Exchanges name to listen for notifications. (multi valued)
#http_control_exchanges = nova
#http_control_exchanges = glance
#http_control_exchanges = neutron
#http_control_exchanges = cinder
# Exchange name for Neutron notifications. (string value)
# Deprecated group/name - [DEFAULT]/quantum_control_exchange
#neutron_control_exchange = neutron
# Allow novaclient's debug log output. (boolean value)
#nova_http_log_debug = false
# Swift reseller prefix. Must be on par with reseller_prefix in proxy-
# server.conf. (string value)
#reseller_prefix = AUTH_
# Configuration file for pipeline definition. (string value)
#pipeline_cfg_file = pipeline.yaml
# Configuration file for event pipeline definition. (string value)
#event_pipeline_cfg_file = event_pipeline.yaml
# Refresh Pipeline configuration on-the-fly. (boolean value)
#refresh_pipeline_cfg = false
# Refresh Event Pipeline configuration on-the-fly. (boolean value)
#refresh_event_pipeline_cfg = false
# Polling interval for pipeline file configuration in seconds.
# (integer value)
#pipeline_polling_interval = 20
# Source for samples emitted on this instance. (string value)
# Deprecated group/name - [DEFAULT]/counter_source
#sample_source = openstack
# Name of this node, which must be valid in an AMQP key. Can be an
# opaque identifier. For ZeroMQ only, must be a valid host name, FQDN,
# or IP address. (string value)
# Timeout seconds for HTTP requests. Set it to None to disable
# timeout. (integer value)
#http_timeout = 600
http_timeout = 600
# DEPRECATED - Database connection string. (string value)
#database_connection = <None>
# Path to the rootwrap configuration file touse for running commands
# as root (string value)
#rootwrap_config = /etc/ceilometer/rootwrap.conf
#
# From oslo.log
#
# Print debugging output (set logging level to DEBUG instead of
# default INFO level). (boolean value)
#debug = false
debug = False
# If set to false, will disable INFO logging level, making WARNING the
# default. (boolean value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#verbose = true
verbose = False
# The name of a logging configuration file. This file is appended to
# any existing logging configuration files. For details about logging
# configuration files, see the Python logging module documentation.
# (string value)
# Deprecated group/name - [DEFAULT]/log_config
#log_config_append = <None>
# DEPRECATED. A logging.Formatter log message format string which may
# use any of the available logging.LogRecord attributes. This option
# is deprecated.  Please use logging_context_format_string and
# logging_default_format_string instead. (string value)
#log_format = <None>
# Format string for %%(asctime)s in log records. Default: %(default)s
# . (string value)
#log_date_format = %Y-%m-%d %H:%M:%S
# (Optional) Name of log file to output to. If no default is set,
# logging will go to stdout. (string value)
# Deprecated group/name - [DEFAULT]/logfile
#log_file = <None>
# (Optional) The base directory used for relative --log-file paths.
# (string value)
# Deprecated group/name - [DEFAULT]/logdir
#log_dir = <None>
log_dir = /var/log/ceilometer
# Use syslog for logging. Existing syslog format is DEPRECATED and
# will be changed later to honor RFC5424. (boolean value)
#use_syslog = false
use_syslog = False
# (Optional) Enables or disables syslog rfc5424 format for logging. If
# enabled, prefixes the MSG part of the syslog message with APP-NAME
# (RFC5424). The format without the APP-NAME is deprecated in Kilo,
# and will be removed in Mitaka, along with this option. (boolean
# value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#use_syslog_rfc_format = true
# Syslog facility to receive log lines. (string value)
#syslog_log_facility = LOG_USER
syslog_log_facility = LOG_USER
# Log output to standard error. (boolean value)
#use_stderr = true
use_stderr = True
# Format string to use for log messages with context. (string value)
#logging_context_format_string = %(asctime)s.%(msecs)03d %(process)d %(levelname)s %(name)s [%(request_id)s %(user_identity)s] %(instance)s%(message)s
# Format string to use for log messages without context. (string
# value)
#logging_default_format_string = %(asctime)s.%(msecs)03d %(process)d %(levelname)s %(name)s [-] %(instance)s%(message)s
# Data to append to log format when level is DEBUG. (string value)
#logging_debug_format_suffix = %(funcName)s %(pathname)s:%(lineno)d
# Prefix each line of exception output with this format. (string
# value)
#logging_exception_prefix = %(asctime)s.%(msecs)03d %(process)d ERROR %(name)s %(instance)s
# List of logger=LEVEL pairs. (list value)
#default_log_levels = amqp=WARN,amqplib=WARN,boto=WARN,qpid=WARN,sqlalchemy=WARN,suds=INFO,oslo.messaging=INFO,iso8601=WARN,requests.packages.urllib3.connectionpool=WARN,urllib3.connectionpool=WARN,websocket=WARN,requests.packages.urllib3.util.retry=WARN,urllib3.util.retry=WARN,keystonemiddleware=WARN,routes.middleware=WARN,stevedore=WARN,taskflow=WARN
# Enables or disables publication of error events. (boolean value)
#publish_errors = false
# The format for an instance that is passed with the log message.
# (string value)
#instance_format = "[instance: %(uuid)s] "
# The format for an instance UUID that is passed with the log message.
# (string value)
#instance_uuid_format = "[instance: %(uuid)s] "
# Enables or disables fatal status of deprecations. (boolean value)
#fatal_deprecations = false
#
# From oslo.messaging
#
# Size of RPC connection pool. (integer value)
# Deprecated group/name - [DEFAULT]/rpc_conn_pool_size
#rpc_conn_pool_size = 30
# ZeroMQ bind address. Should be a wildcard (*), an ethernet
# interface, or IP. The "host" option should point or resolve to this
# address. (string value)
#rpc_zmq_bind_address = *
# MatchMaker driver. (string value)
#rpc_zmq_matchmaker = local
# ZeroMQ receiver listening port. (integer value)
#rpc_zmq_port = 9501
# Number of ZeroMQ contexts, defaults to 1. (integer value)
#rpc_zmq_contexts = 1
# Maximum number of ingress messages to locally buffer per topic.
# Default is unlimited. (integer value)
#rpc_zmq_topic_backlog = <None>
# Directory for holding IPC sockets. (string value)
#rpc_zmq_ipc_dir = /var/run/openstack
# Name of this node. Must be a valid hostname, FQDN, or IP address.
# Must match "host" option, if running Nova. (string value)
#rpc_zmq_host = localhost
# Seconds to wait before a cast expires (TTL). Only supported by
# impl_zmq. (integer value)
#rpc_cast_timeout = 30
# Heartbeat frequency. (integer value)
#matchmaker_heartbeat_freq = 300
# Heartbeat time-to-live. (integer value)
#matchmaker_heartbeat_ttl = 600
# Size of executor thread pool. (integer value)
# Deprecated group/name - [DEFAULT]/rpc_thread_pool_size
#executor_thread_pool_size = 64
# The Drivers(s) to handle sending notifications. Possible values are
# messaging, messagingv2, routing, log, test, noop (multi valued)
#notification_driver =
# AMQP topic used for OpenStack notifications. (list value)
# Deprecated group/name - [rpc_notifier2]/topics
#notification_topics = notifications
notification_topics = notifications
# Seconds to wait for a response from a call. (integer value)
#rpc_response_timeout = 60
# A URL representing the messaging driver to use and its full
# configuration. If not set, we fall back to the rpc_backend option
# and driver specific configuration. (string value)
#transport_url = <None>
# The messaging driver to use, defaults to rabbit. Other drivers
# include qpid and zmq. (string value)
#rpc_backend = rabbit
rpc_backend = rabbit
# The default exchange under which topics are scoped. May be
# overridden by an exchange name specified in the transport_url
# option. (string value)
#control_exchange = openstack
#
# From oslo.service.service
#
# Enable eventlet backdoor.  Acceptable values are 0, <port>, and
# <start>:<end>, where 0 results in listening on a random tcp port
# number; <port> results in listening on the specified port number
# (and not enabling backdoor if that port is in use); and
# <start>:<end> results in listening on the smallest unused port
# number within the specified range of port numbers.  The chosen port
# is displayed in the service's log file. (string value)
#backdoor_port = <None>
# Enables or disables logging values of all registered options when
# starting a service (at DEBUG level). (boolean value)
#log_options = true
meter_dispatcher=database
event_dispatcher=database
[alarm]
#
# From ceilometer
#
# SSL Client certificate for REST notifier. (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#rest_notifier_certificate_file =
# SSL Client private key for REST notifier. (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#rest_notifier_certificate_key =
# Whether to verify the SSL Server certificate when calling alarm
# action. (boolean value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#rest_notifier_ssl_verify = true
# Number of retries for REST notifier (integer value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#rest_notifier_max_retries = 0
# Period of evaluation cycle, should be >= than configured pipeline
# interval for collection of underlying meters. (integer value)
# Deprecated group/name - [alarm]/threshold_evaluation_interval
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#evaluation_interval = 60
evaluation_interval = 60
# The topic that ceilometer uses for alarm notifier messages. (string
# value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#notifier_rpc_topic = alarm_notifier
# URL to Gnocchi. (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#gnocchi_url = http://localhost:8041
# Record alarm change events. (boolean value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#record_history = true
record_history = True
# Maximum number of alarms defined for a user. (integer value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#user_alarm_quota = <None>
# Maximum number of alarms defined for a project. (integer value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#project_alarm_quota = <None>
# Maximum count of actions for each state of an alarm, non-positive
# number means no limit. (integer value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#alarm_max_actions = -1
evaluation_service=ceilometer.alarm.service.SingletonAlarmService
partition_rpc_topic=alarm_partition_coordination
[api]
#
# From ceilometer
#
# The port for the ceilometer API server. (integer value)
# Minimum value: 1
# Maximum value: 65535
# Deprecated group/name - [DEFAULT]/metering_api_port
#port = 8777
port = 8777
# The listen IP for the ceilometer API server. (string value)
#host = 0.0.0.0
host = 192.0.2.10
# Toggle Pecan Debug Middleware. (boolean value)
#pecan_debug = false
# Default maximum number of items returned by API request. (integer
# value)
# Minimum value: 1
#default_api_return_limit = 100
# Number of workers for api, default value is 1. (integer value)
# Minimum value: 1
# Deprecated group/name - [DEFAULT]/api_workers
#workers = 1
[central]
#
# From ceilometer
#
# To reduce polling agent load, samples are sent to the notification
# agent in a batch. To gain higher throughput at the cost of load set
# this to False. (boolean value)
#batch_polled_samples = true
# To reduce large requests at same time to Nova or other components
# from different compute agents, shuffle start time of polling task.
# (integer value)
#shuffle_time_before_polling_task = 0
[collector]
#
# From ceilometer
#
# Address to which the UDP socket is bound. Set to an empty string to
# disable. (string value)
#udp_address = 0.0.0.0
udp_address = 0.0.0.0
# Port to which the UDP socket is bound. (integer value)
# Minimum value: 1
# Maximum value: 65535
#udp_port = 4952
udp_port = 4952
# Requeue the sample on the collector sample queue when the collector
# fails to dispatch it. This is only valid if the sample come from the
# notifier publisher. (boolean value)
#requeue_sample_on_dispatcher_error = false
# Requeue the event on the collector event queue when the collector
# fails to dispatch it. (boolean value)
#requeue_event_on_dispatcher_error = false
# Enable the RPC functionality of collector. This functionality is now
# deprecated in favour of notifier publisher and queues. (boolean
# value)
#enable_rpc = false
# Number of workers for collector service. default value is 1.
# (integer value)
# Minimum value: 1
# Deprecated group/name - [DEFAULT]/collector_workers
#workers = 1
[compute]
#
# From ceilometer
#
# Enable work-load partitioning, allowing multiple compute agents to
# be run simultaneously. (boolean value)
#workload_partitioning = false
[coordination]
#
# From ceilometer
#
# The backend URL to use for distributed coordination. If left empty,
# per-deployment central agent and per-host compute agent won't do
# workload partitioning and will only function correctly if a single
# instance of that service is running. (string value)
#backend_url = <None>
backend_url = redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/
# Number of seconds between heartbeats for distributed coordination.
# (floating point value)
#heartbeat = 1.0
# Number of seconds between checks to see if group membership has
# changed (floating point value)
#check_watchers = 10.0
[database]
#
# From ceilometer
#
# Number of seconds that samples are kept in the database for (<= 0
# means forever). (integer value)
# Deprecated group/name - [database]/time_to_live
#metering_time_to_live = -1
metering_time_to_live = -1
# Number of seconds that events are kept in the database for (<= 0
# means forever). (integer value)
#event_time_to_live = -1
event_time_to_live = -1
# The connection string used to connect to the metering database. (if
# unset, connection is used) (string value)
#metering_connection = <None>
# The connection string used to connect to the alarm database. (if
# unset, connection is used) (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#alarm_connection = <None>
# Number of seconds that alarm histories are kept in the database for
# (<= 0 means forever). (integer value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#alarm_history_time_to_live = -1
alarm_history_time_to_live = -1
# The connection string used to connect to the event database. (if
# unset, connection is used) (string value)
#event_connection = <None>
# The max length of resources id in DB2 nosql, the value should be
# larger than len(hostname) * 2 as compute node's resource id is
# <hostname>_<nodename>. (integer value)
#db2nosql_resource_id_maxlen = 512
# The name of the replica set which is used to connect to MongoDB
# database. Add "?replicaSet=myreplicatset" in your connection URI
# instead. (string value)
# This option is deprecated for removal.
# Its value may be silently ignored in the future.
#mongodb_replica_set =
#
# From oslo.db
#
# The file name to use with SQLite. (string value)
# Deprecated group/name - [DEFAULT]/sqlite_db
#sqlite_db = oslo.sqlite
# If True, SQLite uses synchronous mode. (boolean value)
# Deprecated group/name - [DEFAULT]/sqlite_synchronous
#sqlite_synchronous = true
# The back end to use for the database. (string value)
# Deprecated group/name - [DEFAULT]/db_backend
#backend = sqlalchemy
# The SQLAlchemy connection string to use to connect to the database.
# (string value)
# Deprecated group/name - [DEFAULT]/sql_connection
# Deprecated group/name - [DATABASE]/sql_connection
# Deprecated group/name - [sql]/connection
#connection = <None>
connection = mongodb://192.0.2.11:27017,192.0.2.10:27017,192.0.2.12:27017/ceilometer?replicaSet=tripleo
# The SQLAlchemy connection string to use to connect to the slave
# database. (string value)
#slave_connection = <None>
# The SQL mode to be used for MySQL sessions. This option, including
# the default, overrides any server-set SQL mode. To use whatever SQL
# mode is set by the server configuration, set this to no value.
# Example: mysql_sql_mode= (string value)
#mysql_sql_mode = TRADITIONAL
# Timeout before idle SQL connections are reaped. (integer value)
# Deprecated group/name - [DEFAULT]/sql_idle_timeout
# Deprecated group/name - [DATABASE]/sql_idle_timeout
# Deprecated group/name - [sql]/idle_timeout
#idle_timeout = 3600
idle_timeout = 3600
# Minimum number of SQL connections to keep open in a pool. (integer
# value)
# Deprecated group/name - [DEFAULT]/sql_min_pool_size
# Deprecated group/name - [DATABASE]/sql_min_pool_size
#min_pool_size = 1
min_pool_size = 1
# Maximum number of SQL connections to keep open in a pool. (integer
# value)
# Deprecated group/name - [DEFAULT]/sql_max_pool_size
# Deprecated group/name - [DATABASE]/sql_max_pool_size
#max_pool_size = <None>
max_pool_size = 10
# Maximum number of database connection retries during startup. Set to
# -1 to specify an infinite retry count. (integer value)
# Deprecated group/name - [DEFAULT]/sql_max_retries
# Deprecated group/name - [DATABASE]/sql_max_retries
#max_retries = 10
max_retries = 10
# Interval between retries of opening a SQL connection. (integer
# value)
# Deprecated group/name - [DEFAULT]/sql_retry_interval
# Deprecated group/name - [DATABASE]/reconnect_interval
#retry_interval = 10
retry_interval = 10
# If set, use this value for max_overflow with SQLAlchemy. (integer
# value)
# Deprecated group/name - [DEFAULT]/sql_max_overflow
# Deprecated group/name - [DATABASE]/sqlalchemy_max_overflow
#max_overflow = <None>
max_overflow = 20
# Verbosity of SQL debugging information: 0=None, 100=Everything.
# (integer value)
# Deprecated group/name - [DEFAULT]/sql_connection_debug
#connection_debug = 0
# Add Python stack traces to SQL as comment strings. (boolean value)
# Deprecated group/name - [DEFAULT]/sql_connection_trace
#connection_trace = false
# If set, use this value for pool_timeout with SQLAlchemy. (integer
# value)
# Deprecated group/name - [DATABASE]/sqlalchemy_pool_timeout
#pool_timeout = <None>
# Enable the experimental use of database reconnect on connection
# lost. (boolean value)
#use_db_reconnect = false
# Seconds between retries of a database transaction. (integer value)
#db_retry_interval = 1
# If True, increases the interval between retries of a database
# operation up to db_max_retry_interval. (boolean value)
#db_inc_retry_interval = true
# If db_inc_retry_interval is set, the maximum seconds between retries
# of a database operation. (integer value)
#db_max_retry_interval = 10
# Maximum retries in case of connection error or deadlock error before
# error is raised. Set to -1 to specify an infinite retry count.
# (integer value)
#db_max_retries = 20
time_to_live=-1
[dispatcher_file]
#
# From ceilometer
#
# Name and the location of the file to record meters. (string value)
#file_path = <None>
# The max size of the file. (integer value)
#max_bytes = 0
# The max number of the files to keep. (integer value)
#backup_count = 0
[event]
#
# From ceilometer
#
# Configuration file for event definitions. (string value)
#definitions_cfg_file = event_definitions.yaml
# Drop notifications if no event definition matches. (Otherwise, we
# convert them with just the default traits) (boolean value)
#drop_unmatched_notifications = false
# Store the raw notification for select priority levels (info and/or
# error). By default, raw details are not captured. (multi valued)
#store_raw =
[exchange_control]
#
# From ceilometer
#
# Exchange name for Heat notifications (string value)
#heat_control_exchange = heat
# Exchange name for Glance notifications. (string value)
#glance_control_exchange = glance
# Exchange name for Magnetodb notifications. (string value)
#magnetodb_control_exchange = magnetodb
# Exchange name for Keystone notifications. (string value)
#keystone_control_exchange = keystone
# Exchange name for Cinder notifications. (string value)
#cinder_control_exchange = cinder
# Exchange name for Data Processing notifications. (string value)
#sahara_control_exchange = sahara
# Exchange name for Swift notifications. (string value)
#swift_control_exchange = swift
# Exchange name for Magnum notifications. (string value)
#magnum_control_exchange = magnum
# Exchange name for DBaaS notifications. (string value)
#trove_control_exchange = trove
# Exchange name for Messaging service notifications. (string value)
#zaqar_control_exchange = zaqar
"""

CEILOMETER_DOC_TEST = """
[DEFAULT]
#
# From ceilometer
http_timeout = 600
debug = False
verbose = False
log_dir = /var/log/ceilometer
meter_dispatcher=database
event_dispatcher=database
[alarm]
evaluation_interval = 60
evaluation_service=ceilometer.alarm.service.SingletonAlarmService
partition_rpc_topic=alarm_partition_coordination
[api]
port = 8777
host = 192.0.2.10
[central]
[collector]
udp_address = 0.0.0.0
udp_port = 4952
[compute]
[coordination]
backend_url = redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/
""".strip()


def test_doc_examples():
    env = {
        'config': ceilometer_conf.CeilometerConf(context_wrap(CEILOMETER_DOC_TEST))
    }
    failed, total = doctest.testmod(ceilometer_conf, globs=env)
    assert failed == 0


def test_match():
    result = ceilometer_conf.CeilometerConf(context_wrap(CEILOMETER_CONTENT))
    assert result.data.get("DEFAULT", "http_timeout") == "600"
    assert result.data.get("DEFAULT", "log_dir") == "/var/log/ceilometer"
    assert result.data.get("api", "host") == "192.0.2.10"

    assert result.data.get("coordination", "backend_url") == \
        "redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/"
    assert result.data.get("database", "metering_time_to_live") == "-1"
    assert result.data.get("database", "connection") == \
        "mongodb://192.0.2.11:27017,192.0.2.10:27017,192.0.2.12:27017/ceilometer?replicaSet=tripleo"



from insights.parsers.auditd_conf import AuditdConf
from insights.tests import context_wrap

# from RHEL 6.8, 7.2
AUDITD_CONF_1 = """
#
# This file controls the configuration of the audit daemon
#

log_file = /var/log/audit/audit.log
log_format = RAW
log_group = root
priority_boost = 4
flush = INCREMENTAL
freq = 20
num_logs = 5
disp_qos = lossy
dispatcher = /sbin/audispd
name_format = NONE
##name = mydomain
max_log_file = 6
max_log_file_action = ROTATE
space_left = 75
space_left_action = SYSLOG
action_mail_acct = root
admin_space_left = 50
admin_space_left_action = SUSPEND
disk_full_action = SUSPEND
disk_error_action = SUSPEND
##tcp_listen_port =
tcp_listen_queue = 5
tcp_max_per_addr = 1
##tcp_client_ports = 1024-65535
tcp_client_max_idle = 0
enable_krb5 = no
krb5_principal = auditd
##krb5_key_file = /etc/audit/audit.key

""".strip()

AUDITD_CONF_2 = """
#comment
# comment
# comment = comment
# comment = comment = comment
#comment=comment
#comment=comment=comment
option_a=value_a
option_b = value_b
option_c= value_c
option_d =value_d
broken_option_e = value_e = value_2_e
broken_option_f=value_f=value_2_f
broken_option_g
option_h = value_h # some comment
option_i = value_i # this must be accessible, even after all these errors
""".strip()

AUDITD_CONF_PATH = "etc/audit/auditd.conf"


def test_constructor():
    context = context_wrap(AUDITD_CONF_1, AUDITD_CONF_PATH)
    result = AuditdConf(context)

    assert "tcp_listen_queue = 5" in result.active_lines_unparsed
    assert "SUSPEND" == result.active_settings["disk_error_action"]
    assert "krb5_key_file" not in result.active_settings
    assert "##krb5_key_file" not in result.active_settings
    assert "/var/log/audit/audit.log" == result.get_active_setting_value("log_file")

    context = context_wrap(AUDITD_CONF_2, AUDITD_CONF_PATH)
    result = AuditdConf(context)

    assert "comment" not in result.active_settings
    assert "broken_option_g" not in result.active_settings
    assert "value_i" == result.get_active_setting_value("option_i")


def test_active_lines_unparsed():
    context = context_wrap(AUDITD_CONF_1, AUDITD_CONF_PATH)
    result = AuditdConf(context)
    test_active_lines = []
    for line in AUDITD_CONF_1.split("\n"):
        if not line.strip().startswith("#"):
            if line.strip():
                test_active_lines.append(line)
    assert test_active_lines == result.active_lines_unparsed


def build_active_settings_expected():
    active_settings = {}
    for line in AUDITD_CONF_1.split("\n"):
        if not line.strip().startswith("#"):
            if line.strip():
                try:
                    key, value = line.split("=", 1)
                    key, value = key.strip(), value.strip()
                except:
                    pass
                else:
                    active_settings[key] = value
    return active_settings


def test_active_settings():
    context = context_wrap(AUDITD_CONF_1, AUDITD_CONF_PATH)
    result = AuditdConf(context)
    active_settings = build_active_settings_expected()
    assert active_settings == result.active_settings


def test_get_active_setting_value():
    context = context_wrap(AUDITD_CONF_1, AUDITD_CONF_PATH)
    result = AuditdConf(context)
    active_settings = build_active_settings_expected()
    for key, value in active_settings.items():
        assert result.get_active_setting_value(key) == value



"""
LdLibraryPath - LD_LIBRARY_PATH of Users
========================================

Parser for parsing the environment variable LD_LIBRARY_PATH of each user

"""

from collections import namedtuple
from insights import parser, Parser
from insights.parsers import SkipException
from insights.specs import Specs

LdLibraryPath = namedtuple('LdLibraryPath', ('user', 'path', 'raw'))
"""namedtuple: Type for storing the LD_LIBRARY_PATH of users"""


@parser(Specs.ld_library_path_of_user)
class UserLdLibraryPath(Parser, list):
    """
    Base class for parsing the ``LD_LIBRARY_PATH`` variable of each regular
    user of the system into a list.

    .. note::

        Currently, only the LD_LIBRARY_PATH SAP users is collected, where the
        username is merged by SID and "adm".

    Typical content looks like::

        sr1adm /usr/sap/RH1/SYS/exe/run:/usr/sap/RH1/SYS/exe/uc/linuxx86_64:/sapdb/clients/RH1/lib
        sr2adm
        rh1adm /usr/sap/RH1/SYS/exe/uc/linuxx86_64:/usr/sap/RH1/SYS/exe/run

    Examples:
        >>> len(ld_lib_path)
        3
        >>> isinstance(ld_lib_path[0].path, list)
        True
        >>> len(ld_lib_path[0].path)
        3
        >>> '/sapdb/clients/RH1/lib' in ld_lib_path[0].path
        True
        >>> ld_lib_path[1].user  # The empty value is kept.
        'sr2adm'
        >>> '' in ld_lib_path[1].path  # The empty value is kept.
        True

    Raises:
        SkipException: When the output is empty or nothing needs to parse.
    """

    def parse_content(self, content):
        llds = []
        for line in content:
            user, _, raw = [s.strip() for s in line.partition(' ')]
            paths = raw
            if raw and raw[0] == raw[-1] and raw[0] in ('\'', '"'):
                paths = raw[1:-1]
            llds.append(LdLibraryPath(user, paths.split(':'), raw))

        if not llds:
            raise SkipException("LD_LIBRARY_PATH not set.")

        self.extend(llds)



#!/usr/bin/env python

from __future__ import print_function

import re
import socket

from insights import rule, make_metadata, run
from insights.specs import Specs
from insights.core import Parser
from insights.core.plugins import parser
from insights.core.dr import set_enabled, load_components
import uuid


def valid_uuid_or_None(s):
    """
    Returns  if `s` is a valid UUID string.
    """
    if not s:
        return None
    try:
        return str(uuid.UUID(s))
    except Exception:
        return None


def valid_ipv4_address_or_None(addr):
    """ str: Returns the input value if it is a valid IPV4 address """
    try:
        socket.inet_pton(socket.AF_INET, addr)
        return addr
    except socket.error:
        return None


def valid_mac_addresses(mac_address_datasources):
    """ list: Return a list of valid mac addresses from a list of datasources """
    valid_addrs = []
    for ds in mac_address_datasources:
        try:
            addr = ds.content[0].strip()
        except Exception:
            continue
        # Only look for addresses in the form 00:00:00:00:00:00
        match = re.match("^([0-9a-f]{2}:){5}[0-9a-f]{2}$", addr)
        if match is not None:
            valid_addrs.append(addr)

    return valid_addrs


@parser(Specs.ip_addresses)
class IPs(Parser):
    """
    Reads the output of ``hostname -I`` and constructs a list of all assigned IP
    addresses. This command should only output IPV4 addresses and should not
    include localhost, but sometimes it does.  The validation function removes
    those from the list.

    Example output::
       192.168.1.71 10.88.0.1 172.17.0.1 172.18.0.1 10.10.121.131

    Resultant data structure::
        [
            "192.168.1.71",
            "10.88.0.1",
            "172.17.0.1",
            "172.18.0.1",
            "10.10.121.131",
        ]
    """
    def parse_content(self, content):
        self.data = list(filter(None, [valid_ipv4_address_or_None(addr) for addr in content[0].rstrip().split()]))


@parser(Specs.subscription_manager_id)
class SubscriptionManagerID(Parser):
    """
    Reads the output of subscription-manager identity and retrieves the UUID

    Example output::
        system identity: 6655c27c-f561-4c99-a23f-f53e5a1ef311
        name: rhel7.localdomain
        org name: 1234567
        org ID: 1234567

    Resultant data::

        6655c27c-f561-4c99-a23f-f53e5a1ef311
    """

    def parse_content(self, content):
        self.data = content[0].split(":")[-1].strip()


def _safe_parse(ds):
    try:
        return ds.content[0]

    except Exception:
        return None


def _filter_falsy(dict_):
    return dict((k, v) for k, v in dict_.items() if v)


@rule(
    optional=[
        Specs.machine_id,
        Specs.etc_machine_id,
        Specs.bios_uuid,
        SubscriptionManagerID,
        IPs,
        Specs.hostname,
        Specs.mac_addresses,
    ]
)
def canonical_facts(
    insights_id, machine_id, bios_uuid, submanid, ips, fqdn, mac_addresses
):

    facts = dict(
        insights_id=valid_uuid_or_None(_safe_parse(insights_id)),
        machine_id=valid_uuid_or_None(_safe_parse(machine_id)),
        bios_uuid=valid_uuid_or_None(_safe_parse(bios_uuid)),
        subscription_manager_id=valid_uuid_or_None(submanid.data if submanid else None),
        ip_addresses=ips.data if ips else [],
        mac_addresses=valid_mac_addresses(mac_addresses) if mac_addresses else [],
        fqdn=_safe_parse(fqdn),
    )

    return make_metadata(**_filter_falsy(facts))


def get_canonical_facts(path=None):
    load_components("insights.specs.default", "insights.specs.insights_archive")

    set_enabled(canonical_facts, True)
    set_enabled(SubscriptionManagerID, True)
    set_enabled(IPs, True)
    br = run(canonical_facts, root=path)
    d = br[canonical_facts]
    del d["type"]
    return d


if __name__ == "__main__":
    import json

    print(json.dumps(get_canonical_facts()))



"""
Cloud Provider
==============

Combiner for Cloud information. It uses the results of the multiple parsers:

* :py:class:`insights.parsers.installed_rpms.InstalledRpms`
* :py:class:`insights.parsers.yum.YumRepoList`
* :py:class:`insights.parsers.dmidecode.DMIDecode`
* :py:class:`insights.parsers.rhsm_conf.RHSMConf`

The combiner uses these parsers determine the Cloud Provider based on a set of
criteria that is unique to each cloud provider.

Examples:
    >>> cp_aws.cloud_provider
    'aws'
    >>> cp_aws.cp_bios_version['aws'] == '4.2.amazon'
    True
    >>> cp_aws.cp_rpms['aws'] == ['rh-amazon-rhui-client-2.2.124-1.el7']
    True
    >>> cp_aws.cp_uuid['aws']
    'EC2F58AF-2DAD-C57E-88C0-A81CB6084290'
    >>> cp_aws.long_name
    'Amazon Web Services'
    >>> cp_azure.cloud_provider
    'azure'
    >>> cp_azure.cp_yum['azure'] == ['rhui-microsoft-azure-rhel7-2.2-74']
    True
    >>> cp_azure.cp_asset_tag['azure']
    '7783-7084-3265-9085-8269-3286-77'
    >>> cp_alibaba.cloud_provider
    'alibaba'
    >>> cp_alibaba.cp_manufacturer['alibaba'] == 'Alibaba Cloud'
    True
    >>> cp_ibm.cp_rhsm_server_hostname['ibm'] == 'host.networklayer.com'
    True

"""

from insights.core.plugins import combiner
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.dmidecode import DMIDecode
from insights.parsers.yum import YumRepoList
from insights.parsers.rhsm_conf import RHSMConf


class CloudProviderInstance(object):
    """
    Class to represent a base cloud provider instance

    Use this base class to derive new cloud provider classes.  In each new cloud
    provider class set the particular values that will be used to detect that
    particular cloud provider.

    Attributes:

        rpm (str): RPM string in lowercase to use when searching for this cloud provider.
        yum (str): Yum repo name string in lowercase to use when searching for this cloud provider.
        bios_vendor_version (str): BIOS vendor version string in lowercase to use when searching
            for this cloud provider.
        manuf (str): Manufacturer string in lowercase to use when searching for this cloud provider.
        asset_tag (str): Asset tag string in lowercase to use when searching for this
            cloud provider.
        uuid (str): UUID string in lowercase to use when searchinf for this cloud provider.
        rhsm_hostname (str): Hostname string in lowercase to use when searching for this
            cloud provider in ``rhsm.conf``.
        cp_bios_vendor (str): BIOS vendor string value found in search for this cloud provider.
        cp_bios_version (str): BIOS version string value found in search for this cloud provider.
        cp_rpms (list): List of RPM string values found in search for this cloud provider.
        cp_yum (list): List of Yum repo name string values found in search for this cloud provider.
        cp_asset_tag (str): Asset tag string value found in search for this cloud provider.
        cp_uuid (str): UUID string value found in search for this cloud provider.
        cp_manufacturer (str): Manufacturer string value found in search for this cloud provider.
        cp_rhsm_server_hostname (str): RHSM server hostname string value found in search for
            this cloud provider.

    """
    def __init__(self, rpms=None, dmidcd=None, yum_repos=None, rhsm_cfg=None):
        self._rpms = rpms
        self._dmidcd = dmidcd
        self._yum_repos = yum_repos
        self._rhsm_cfg = rhsm_cfg
        self.rpm = ''
        self.yum = ''
        self.bios_vendor_version = ''
        self.manuf = ''
        self.asset_tag = ''
        self.uuid = ''
        self.rhsm_hostname = ''
        self.cp_bios_vendor = ''
        self.cp_bios_version = ''
        self.cp_rpms = []
        self.cp_yum = []
        self.cp_asset_tag = ''
        self.cp_uuid = ''
        self.cp_manufacturer = ''
        self.cp_rhsm_server_hostname = ''

    def _get_cp_bios_vendor(self, vendor_version):
        """ str: Returns BIOS vendor string if it matches ``vendor_version`` """
        vendor = ''
        if self._dmidcd and self._dmidcd.bios:
            vendor = (
                self._dmidcd.bios.get('vendor')
                if vendor_version and vendor_version in self._dmidcd.bios.get('vendor', '').lower() else ''
            )
        return vendor

    def _get_cp_bios_version(self, vendor_version):
        """ str: Returns BIOS version string if it matches ``vendor_version`` """
        version = ''
        if self._dmidcd and self._dmidcd.bios:
            version = (
                self._dmidcd.bios.get('version')
                if vendor_version and vendor_version in self._dmidcd.bios.get('version', '').lower() else ''
            )
        return version

    def _get_rpm_cp_info(self, rpm):
        """ list: Returns list of RPMs matching ``rpm`` """
        found_rpms = []
        if self._rpms:
            for key, val in self._rpms.packages.items():
                for v in val:
                    if rpm and rpm in v.package.lower():
                        found_rpms.append(v.package)
        return found_rpms

    def _get_cp_from_manuf(self, manuf):
        """ str: Returns manufacturer string if it matches ``manuf`` """
        manufacturer = ''
        if self._dmidcd and self._dmidcd.system_info:
            manufacturer = (
                self._dmidcd.system_info.get('manufacturer')
                if manuf == self._dmidcd.system_info.get('manufacturer', '').lower() else ''
            )
        return manufacturer

    def _get_cp_from_yum(self, repo_name):
        """ list: Returns list of Yum repos matching ``repo_name`` """
        found_repos = []
        if self._yum_repos and hasattr(self._yum_repos, 'data'):
            found_repos = [
                repo.get('id').lower()
                for repo in self._yum_repos.data
                if repo_name and repo_name in repo.get('id', '').lower()
            ]
        return found_repos

    def _get_cp_from_rhsm_conf(self, rhsm_server_hostname):
        """ str: Returns rhsm server hostname string if it matches ``rhsm_server_hostname`` """
        server_hostname = ''
        if self._rhsm_cfg and 'server' in self._rhsm_cfg and 'hostname' in self._rhsm_cfg['server']:
            hostname = self._rhsm_cfg.get('server', 'hostname')
            if hostname and hostname.lower().strip().endswith(rhsm_server_hostname):
                server_hostname = hostname
        return server_hostname

    def _get_cp_from_asset_tag(self, asset_tag):
        """ str: Returns asset tag string if it matches ``asset_tag`` """
        tag = ''
        if self._dmidcd and hasattr(self._dmidcd, 'data'):
            ch_info = self._dmidcd.data.get('chassis_information', [])
            if ch_info:
                tag = ch_info[0].get('asset_tag') if asset_tag and asset_tag == ch_info[0].get('asset_tag', '') else ''
        return tag

    def _get_cp_from_uuid(self, uuid):
        """ str: Returns UUID string if it matches ``uuid`` """
        found_uuid = ''
        if self._dmidcd and self._dmidcd.system_info:
            found_uuid = (
                self._dmidcd.system_info.get('uuid')
                if uuid and self._dmidcd.system_info.get('uuid', '').lower().strip().startswith(uuid) else ''
            )
        return found_uuid

    @property
    def name(self):
        """ str: Short cloud provider class name or ID """
        return self._NAME

    @property
    def long_name(self):
        """ str: Long cloud provider name """
        return self._LONG_NAME


class GoogleCloudProvider(CloudProviderInstance):
    """
    Class to identify Google Cloud provider

    Google CP can be identified by RPM and BIOS vendor/version
    """
    _NAME = 'google'
    _LONG_NAME = 'Google Cloud'

    def __init__(self, *args, **kwargs):
        super(GoogleCloudProvider, self).__init__(*args, **kwargs)
        self.rpm = 'google-rhui-client'
        self.bios_vendor_version = 'google'
        self.cp_bios_vendor = self._get_cp_bios_vendor(self.bios_vendor_version)
        self.cp_bios_version = self._get_cp_bios_version(self.bios_vendor_version)
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)


class AlibabaCloudProvider(CloudProviderInstance):
    """
    Class to identify Alibaba Cloud provider

    Alibaba CP can be identified by manufacturer
    """
    _NAME = 'alibaba'
    _LONG_NAME = 'Alibaba Cloud'

    def __init__(self, *args, **kwargs):
        super(AlibabaCloudProvider, self).__init__(*args, **kwargs)
        self.manuf = 'alibaba cloud'
        self.cp_manufacturer = self._get_cp_from_manuf(self.manuf)


class AmazonCloudProvider(CloudProviderInstance):
    """
    Class to identify Amazon Cloud provider

    Amazon CP can be identified by RPM, BIOS verndor/version,
    and system UUID
    """
    _NAME = 'aws'
    _LONG_NAME = 'Amazon Web Services'

    def __init__(self, *args, **kwargs):
        super(AmazonCloudProvider, self).__init__(*args, **kwargs)
        self.rpm = 'rh-amazon-rhui-client'
        self.bios_vendor_version = 'amazon'
        self.uuid = 'ec2'
        self.asset_tag = 'Amazon EC2'
        self.cp_bios_vendor = self._get_cp_bios_vendor(self.bios_vendor_version)
        self.cp_bios_version = self._get_cp_bios_version(self.bios_vendor_version)
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        self.cp_uuid = self._get_cp_from_uuid(self.uuid)
        self.cp_asset_tag = self._get_cp_from_asset_tag(self.asset_tag)


class AzureCloudProvider(CloudProviderInstance):
    """
    Class to identify Azure Cloud provider

    Azure CP can be identified by RPM, Yum repo, and system asset tag
    """
    _NAME = 'azure'
    _LONG_NAME = 'Microsoft Azure'

    def __init__(self, *args, **kwargs):
        super(AzureCloudProvider, self).__init__(*args, **kwargs)
        self.rpm = 'walinuxagent'
        self.yum = 'rhui-microsoft-azure'
        self.asset_tag = '7783-7084-3265-9085-8269-3286-77'
        self.cp_asset_tag = self._get_cp_from_asset_tag(self.asset_tag)
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        self.cp_yum = self._get_cp_from_yum(self.yum)


class IBMCloudProvider(CloudProviderInstance):
    """
    Class to identify IBM Cloud provider

    IBM CP can be identified by rhsm.conf server hostname setting
    """
    _NAME = 'ibm'
    _LONG_NAME = 'IBM Cloud'

    def __init__(self, *args, **kwargs):
        super(IBMCloudProvider, self).__init__(*args, **kwargs)
        self.rhsm_server_hostname = 'networklayer.com'
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        self.cp_yum = self._get_cp_from_yum(self.yum)
        self.cp_rhsm_server_hostname = self._get_cp_from_rhsm_conf(self.rhsm_server_hostname)


@combiner([InstalledRpms, DMIDecode, YumRepoList, RHSMConf])
class CloudProvider(object):
    """
    Combiner class to provide cloud vendor facts

    Attributes:
        cp_bios_vendor (dict): Dictionary containing a value , for each provider,
            of Bios vendor used to determine cloud provider. Each providers value will be
            empty if none found
        cp_bios_version (dict): Dictionary containing a value, for each provider,
            of Bios version used to determine cloud provider. Each providers value will be
            empty if none found
        cp_rpms (dict): Dictionary containing a list, for each provider, of rpm information
            used to determine cloud provider. Each providers list will be empty if no matches
            found
        cp_yum (dict): Dictionary containing a list, for each provider, of yum repo information
            used to determine cloud provider. Each providers list will be empty if no matches
            found
        cp_asset_tag (dict): Dictionary containing a value, for each provider, of rpm information
            used to determine cloud provider. Each providers value will be empty if no matches
            found
        cp_uuid (dict): Dictionary containing a value, for each provider, of uuid information
            used to determine cloud provider. Each providers value will be empty if no matches
            are found
        cp_manufacturer (dict): Dictionary containing a value, for each provider, of system
            information used to determine cloud provider. Provider value will be empty if no
            matches are found.
        cp_rhsm_server_hostname (dict): Dictionary containing a value, for each provider,
            of rhsm.conf server hostnames.  Value will be empty if no matches are found.
        cloud_provider (str): String representing the cloud provider that was detected.
            If none are detected then it will have the default value `None`.
    """
    ALIBABA = AlibabaCloudProvider._NAME
    """Alibaba Cloud Provider short name"""

    AWS = AmazonCloudProvider._NAME
    """AWS Cloud Provider short name"""

    AZURE = AzureCloudProvider._NAME
    """AZURE Cloud Provider short name"""

    GOOGLE = GoogleCloudProvider._NAME
    """GOOGLE Cloud Provider short name"""

    IBM = IBMCloudProvider._NAME
    """IBM Cloud Provider short name"""

    # Add any new cloud provider classes to this list
    _CLOUD_PROVIDER_CLASSES = [
        GoogleCloudProvider,
        AlibabaCloudProvider,
        AmazonCloudProvider,
        AzureCloudProvider,
        IBMCloudProvider,
    ]

    def __init__(self, rpms, dmidcd, yrl, rhsm_cfg):
        self._cp_objects = dict([
            (cls._NAME, cls(rpms=rpms, dmidcd=dmidcd, yum_repos=yrl, rhsm_cfg=rhsm_cfg))
            for cls in self._CLOUD_PROVIDER_CLASSES
        ])
        self.cp_bios_vendor = dict([(name, cp.cp_bios_vendor) for name, cp in self._cp_objects.items()])
        self.cp_bios_version = dict([(name, cp.cp_bios_version) for name, cp in self._cp_objects.items()])
        self.cp_rpms = dict([(name, cp.cp_rpms) for name, cp in self._cp_objects.items()])
        self.cp_yum = dict([(name, cp.cp_yum) for name, cp in self._cp_objects.items()])
        self.cp_asset_tag = dict([(name, cp.cp_asset_tag) for name, cp in self._cp_objects.items()])
        self.cp_uuid = dict([(name, cp.cp_uuid) for name, cp in self._cp_objects.items()])
        self.cp_manufacturer = dict([(name, cp.cp_manufacturer) for name, cp in self._cp_objects.items()])
        self.cp_rhsm_server_hostname = dict([(name, cp.cp_rhsm_server_hostname) for name, cp in self._cp_objects.items()])
        self.cloud_provider = self._select_provider()

    def _select_provider(self):
        """
        This method provides the logic to identify which cloud provider is present.

        If new data sources and/or cloud providers are added you must add logic here to
        identify the new cloud provider.

        Returns:
            str: Returns the name of the cloud provider, corresponds to ``name`` property
                in cloud provider classes.  If no cloud provider is identified, ``None`` is returned
        """
        # Check bios vendor first
        if self._cp_objects[self.AWS].cp_bios_vendor:
            return self.AWS
        elif self._cp_objects[self.GOOGLE].cp_bios_vendor:
            return self.GOOGLE
        elif self._cp_objects[self.AZURE].cp_bios_vendor:
            return self.AZURE

        # Specific vendor not detected, so check bios version
        if self._cp_objects[self.AWS].cp_bios_version:
            return self.AWS
        elif self._cp_objects[self.GOOGLE].cp_bios_version:
            return self.GOOGLE
        elif self._cp_objects[self.AZURE].cp_bios_version:
            return self.AZURE

        # BIOS vendor and version not detected check for RPMs
        if self._cp_objects[self.AWS].cp_rpms:
            return self.AWS
        elif self._cp_objects[self.GOOGLE].cp_rpms:
            return self.GOOGLE
        elif self._cp_objects[self.AZURE].cp_rpms:
            return self.AZURE

        # No luck, check for other attributes
        if self._cp_objects[self.AZURE].cp_yum or self._cp_objects[self.AZURE].cp_asset_tag:
            return self.AZURE

        if self._cp_objects[self.AWS].cp_uuid and self._cp_objects[self.AWS].cp_asset_tag:
            return self.AWS

        if self._cp_objects[self.ALIBABA].cp_manufacturer:
            return self.ALIBABA

        if self._cp_objects[self.IBM].cp_rhsm_server_hostname:
            return self.IBM

        return None

    @property
    def long_name(self):
        """ str: Return long name for the specific cloud provider, or ``None`` if no cloud provider """
        return self._cp_objects[self.cloud_provider].long_name if self.cloud_provider is not None else None



from insights.tests import context_wrap
from insights.parsers.cpu_vulns import CpuVulns
from insights.combiners import cpu_vulns_all
from insights.combiners.cpu_vulns_all import CpuVulnsAll
from insights.parsers import SkipComponent
import doctest
import pytest

IMPUT_MELTDOWN_EMPTY = """
""".strip()

INPUT_MELTDOWN = """
Mitigation: PTI
""".strip()

INPUT_SPECTRE_V1 = """
Mitigation: Load fences
""".strip()

INPUT_SPECTRE_V2 = """
Mitigation: Full generic retpoline, IBPB: conditional, IBRS_FW, STIBP: conditional, RSB filling
""".strip()

INPUT_SPEC_STORE_BYPASS = """
Mitigation: Speculative Store Bypass disabled
""".strip()

INPUT_SMT = """
Mitigation: PTE Inversion; VMX: conditional cache flushes, SMT vulnerable
""".strip()

INPUT_MDS = """
Vulnerable: Clear CPU buffers attempted, no microcode; SMT vulnerable
""".strip()

INPUT0 = context_wrap(INPUT_MELTDOWN, path='')
INPUT1 = context_wrap(INPUT_MELTDOWN, path='/sys/devices/system/cpu/vulnerabilities/meltdown')
INPUT2 = context_wrap(INPUT_SPECTRE_V1, path='/sys/devices/system/cpu/vulnerabilities/spectre_v1')
INPUT3 = context_wrap(INPUT_SPECTRE_V2, path='/sys/devices/system/cpu/vulnerabilities/spectre_v2')
INPUT4 = context_wrap(INPUT_SPEC_STORE_BYPASS, path='/sys/devices/system/cpu/vulnerabilities/spec_store_bypass')
INPUT5 = context_wrap(INPUT_SMT, path='/sys/devices/system/cpu/vulnerabilities/l1tf')
INPUT6 = context_wrap(INPUT_MDS, path='/sys/devices/system/cpu/vulnerabilities/mds')

parser0 = CpuVulns(INPUT0)
parser1 = CpuVulns(INPUT1)
parser2 = CpuVulns(INPUT2)
parser3 = CpuVulns(INPUT3)
parser4 = CpuVulns(INPUT4)
parser5 = CpuVulns(INPUT5)
parser6 = CpuVulns(INPUT6)


def test_values_comb_meltdown():
    obj = CpuVulnsAll([parser1, parser2, parser3])
    assert 'meltdown' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'spectre_v1': 'Mitigation: Load fences', 'spectre_v2': 'Mitigation: Full generic retpoline, IBPB: conditional, IBRS_FW, STIBP: conditional, RSB filling'}


def test_values_comb_spectre_v1():
    obj = CpuVulnsAll([parser1, parser2])
    assert 'spectre_v1' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'spectre_v1': 'Mitigation: Load fences'}


def test_values_comb_spectre_v2():
    obj = CpuVulnsAll([parser1, parser3])
    assert 'spectre_v2' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'spectre_v2': 'Mitigation: Full generic retpoline, IBPB: conditional, IBRS_FW, STIBP: conditional, RSB filling'}


def test_values_comb_spec_store_bypass():
    obj = CpuVulnsAll([parser1, parser4])
    assert 'spec_store_bypass' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'spec_store_bypass': 'Mitigation: Speculative Store Bypass disabled'}


def test_values_comb_l1tf():
    obj = CpuVulnsAll([parser1, parser5])
    assert 'l1tf' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'l1tf': 'Mitigation: PTE Inversion; VMX: conditional cache flushes, SMT vulnerable'}


def test_values_comb_mds():
    obj = CpuVulnsAll([parser6])
    assert 'mds' in obj
    assert obj == {'mds': 'Vulnerable: Clear CPU buffers attempted, no microcode; SMT vulnerable'}


def test_values_integration():
    obj = CpuVulnsAll([parser1, parser2, parser3, parser4, parser5, parser6])
    assert 'spectre_v1' and 'spec_store_bypass' in obj
    assert obj == {'meltdown': 'Mitigation: PTI', 'spectre_v1': 'Mitigation: Load fences', 'spectre_v2': 'Mitigation: Full generic retpoline, IBPB: conditional, IBRS_FW, STIBP: conditional, RSB filling', 'spec_store_bypass': 'Mitigation: Speculative Store Bypass disabled', 'l1tf': 'Mitigation: PTE Inversion; VMX: conditional cache flushes, SMT vulnerable', 'mds': 'Vulnerable: Clear CPU buffers attempted, no microcode; SMT vulnerable'}


def test_values_exp():
    with pytest.raises(SkipComponent) as pe:
        CpuVulnsAll([parser0])
    assert "Not available data" in str(pe)


def test_x86_enabled_documentation():
    """
    Here we test the examples in the documentation automatically using
    doctest.  We set up an environment which is similar to what a rule
    writer might see - a '/sys/devices/system/cpu/vulnerabilities/*' output
    that has been passed in as a parameter to the rule declaration.
    """

    parser1 = CpuVulns(INPUT1)
    parser2 = CpuVulns(INPUT2)
    env = {'cvb': CpuVulnsAll([parser1, parser2])}
    failed, total = doctest.testmod(cpu_vulns_all, globs=env)
    assert failed == 0



"""
Custom datasources for SAP related specs
"""
from insights.specs import Specs
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.combiners.sap import Sap
from insights.specs.datasources import DEFAULT_SHELL_TIMEOUT


class LocalSpecs(Specs):
    """ Local specs used only by sap datasources """

    @datasource(Sap, HostContext)
    def sap_instance(broker):
        """
        list: List of all SAP Instances.
        """
        sap = broker[Sap]
        return sorted(v for v in sap.values())

    @datasource(sap_instance, HostContext)
    def sap_hana_instance(broker):
        """
        list: List of the SAP HANA Instances.
        """
        sap = broker[LocalSpecs.sap_instance]
        insts = sorted(v for v in sap if v.type == 'HDB')
        if insts:
            return insts
        raise SkipComponent()


@datasource(LocalSpecs.sap_instance, HostContext)
def sap_sid(broker):
    """
    list: List of the SID of all the SAP Instances.
    """
    sap = broker[LocalSpecs.sap_instance]
    return sorted(set(h.sid.lower() for h in sap))


@datasource(LocalSpecs.sap_hana_instance, HostContext)
def sap_hana_sid(broker):
    """
    list: List of the SID of SAP HANA Instances.  """
    hana = broker[LocalSpecs.sap_hana_instance]
    sids = sorted(set(h.sid.lower() for h in hana))
    if sids:
        return sids
    raise SkipComponent()


@datasource(LocalSpecs.sap_hana_instance, HostContext)
def sap_hana_sid_SID_nr(broker):
    """
    list: List of tuples (sid, SID, Nr) of SAP HANA Instances.
    """
    hana = broker[LocalSpecs.sap_hana_instance]
    sids = sorted((h.sid.lower(), h.sid, h.number) for h in hana)
    if sids:
        return sids
    raise SkipComponent()


@datasource(sap_sid, HostContext)
def ld_library_path_of_user(broker):
    """
    list: The list of "Username LD_LIBRARY_PATH", e.g.::

          [
            'sr1adm /usr/sap/RH1/SYS/exe/run:/usr/lib/',
            'sr2adm /usr/sap/RH2/SYS/exe/run',
          ]

    .. note::
        Currently, only Sap users are supported.
    """
    ctx = broker[HostContext]
    llds = []
    for sid in broker[sap_sid]:
        usr = '{0}adm'.format(sid)
        ret, vvs = ctx.shell_out("/bin/su -l {0} -c /bin/env".format(usr), keep_rc=True, timeout=DEFAULT_SHELL_TIMEOUT)
        if ret != 0:
            continue
        for v in vvs:
            if "LD_LIBRARY_PATH=" in v:
                llds.append('{0} {1}'.format(usr, v.split('=', 1)[-1]))
    if llds:
        return DatasourceProvider('\n'.join(llds), relative_path='insights_commands/echo_user_LD_LIBRARY_PATH')
    raise SkipComponent('')



#!/usr/bin/env python
import argparse
import logging

from insights.ocp import analyze


log = logging.getLogger(__name__)

banner = """
Openshift Configuration Explorer

Tutorial: https://github.com/RedHatInsights/insights-core/blob/master/docs/notebooks/Parsr%20Query%20Tutorial.ipynb

conf is the top level configuration. Use conf.get_keys() to see first level keys.

Available Predicates
    lt, le, eq, gt, ge

    isin, contains

    startswith, endswith, matches

    ieq, icontains, istartswith, iendswith

Available Operators
    ~ (not)
    | (or)
    & (and)

Example
    api = conf.where("kind", "KubeAPIServer")
    latest = api.status.latestAvailableRevision.value
    api.status.nodeStatuses.where("currentRevision", ~eq(latest))
"""


def parse_args():
    p = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    p.add_argument("archives", nargs="+", help="Archive or directory to analyze.")
    p.add_argument("-D", "--debug", help="Verbose debug output.", action="store_true")
    p.add_argument("--exclude", default="*.log", help="Glob patterns to exclude separated by commas")
    return p.parse_args()


def parse_exclude(exc):
    return [e.strip() for e in exc.split(",")]


def main():
    args = parse_args()
    archives = args.archives
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    excludes = parse_exclude(args.exclude) if args.exclude else ["*.log"]

    conf = analyze(archives, excludes)  # noqa F841 / unused var

    # import all the built-in predicates
    from insights.parsr.query import (lt, le, eq, gt, ge, isin, contains,  # noqa: F401,F403
            startswith, endswith, ieq, icontains, istartswith, iendswith,
            matches, make_child_query)
    q = make_child_query

    import IPython
    from traitlets.config.loader import Config

    ns = dict(locals())
    ns["analyze"] = analyze
    ns["ALL"] = None
    ns["ANY"] = None

    IPython.core.completer.Completer.use_jedi = False
    c = Config()
    c.TerminalInteractiveShell.banner1 = banner
    IPython.start_ipython([], user_ns=ns, config=c)


if __name__ == "__main__":
    main()



"""
SystemUserDirs - datasource ``system_user_dirs``
================================================

Parser for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.
For more information, see the ``system_user_dirs`` datasource.
"""

from insights import Parser, parser
from insights.specs import Specs


@parser(Specs.system_user_dirs)
class SystemUserDirs(Parser):
    """
    Class for enabling the data from the ``system_user_dirs`` datasource.

    Sample output of this datasource is::

        ["ca-certificates", "kmod", "sssd-ldap"]

    Examples:

        >>> type(system_user_dirs)
        <class 'insights.parsers.system_user_dirs.SystemUserDirs'>
        >>> system_user_dirs.packages
        ['ca-certificates', 'kmod', 'sssd-ldap']
    """

    def parse_content(self, content):
        self.packages = content



import doctest

from insights.parsers import rpm_v_packages
from insights.parsers.rpm_v_packages import RpmVPackages
from insights.tests import context_wrap


TEST_RPM = """
package procps is not installed
..?......  c /etc/sudoers
..?......    /usr/bin/sudo
..?......    /usr/bin/sudoreplay
missing     /var/db/sudo/lectured (Permission denied)
"""

TEST_RPM_2 = """
package procps is not installed
S.5....T.  c /etc/sudoers
S.5....T.  c /etc/chrony.conf
"""


def test_rpm_empty():
    rpm_pkgs = RpmVPackages(context_wrap([]))
    assert rpm_pkgs.packages_list == []


def test_rpm():
    line_1 = {'attributes': None, 'file': None,
              'line': 'package procps is not installed', 'mark': None}
    line_2 = {'attributes': '..?......', 'file': '/etc/sudoers',
              'line': '..?......  c /etc/sudoers', 'mark': 'c'}
    line_3 = {'attributes': '..?......', 'file': '/usr/bin/sudo',
              'line': '..?......    /usr/bin/sudo', 'mark': None}
    line_4 = {'attributes': '..?......', 'file': '/usr/bin/sudoreplay',
              'line': '..?......    /usr/bin/sudoreplay', 'mark': None}
    line_5 = {'attributes': None, 'file': None,
              'line': 'missing     /var/db/sudo/lectured (Permission denied)', 'mark': None}

    rpm_pkgs = RpmVPackages(context_wrap(TEST_RPM))
    assert rpm_pkgs.packages_list[0] == line_1
    assert rpm_pkgs.packages_list[1] == line_2
    assert rpm_pkgs.packages_list[2] == line_3
    assert rpm_pkgs.packages_list[3] == line_4
    assert rpm_pkgs.packages_list[4] == line_5

    rpm_pkgs_2 = RpmVPackages(context_wrap(TEST_RPM_2))
    assert rpm_pkgs_2.packages_list[2].get('file', None) == '/etc/chrony.conf'


def test_doc_examples():
    env = {
        "rpm_v_packages": RpmVPackages(context_wrap(TEST_RPM))
    }
    failed, total = doctest.testmod(rpm_v_packages, globs=env)
    assert failed == 0



"""
Custom datasource for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.
"""

import grp
import pwd

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """
    Local spec used only by the system_user_dirs datasource
    """
    rpm_args = simple_command(
        'rpm -qa --qf="[%{=NAME}; %{FILEMODES:perms}; %{FILEUSERNAME}; %{FILEGROUPNAME}\n]"'
    )


def get_shells():
    """
    Returns all full pathnames of valid login shells without nologins.
    """
    with open("/etc/shells") as file:
        return [line.strip() for line in file if "nologin" not in line]


def get_users():
    """
    Returns all users with shell specified in get_shells() except for root.
    """
    shells = get_shells()
    users = []

    for user in pwd.getpwall():
        name = user[0]
        shell = user[6]

        if name == "root" or (shell not in shells):
            continue
        users.append(name)
    return users


def get_groups(users):
    """
    Returns all groups for users specified in get_users().
    Every user has at least one group with its name.
    """
    groups = []

    for group in grp.getgrall():
        group_name = group[0]
        user_list = group[3]

        if group_name in users:
            groups.append(group_name)  # group for an user of interest
        for user in user_list:
            if user.strip() in users:
                groups.append(group_name)  # add groups containing a user
    return groups


@datasource(LocalSpecs.rpm_args, HostContext)
def system_user_dirs(broker):
    r"""
    Custom datasource for CVE-2021-35937, CVE-2021-35938, and CVE-2021-35939.

    It collects package names from the ``rpm -qa --qf="[%{=NAME}; %{FILEMODES:perms};
    %{FILEUSERNAME}; %{FILEGROUPNAME}\n]" command``, if the package has
    at least one directory which is writable by a specific user/group or the others.

    Raises:
        SkipComponent: Raised if no data is available

    Returns:
        List[str]: Sorted list of package names
    """
    users = get_users()
    groups = get_groups(users)
    packages = set()
    content = broker[LocalSpecs.rpm_args].content

    if not content or "command not found" in content[0]:
        raise SkipComponent

    for line in content:
        name, perms, user, group = line.split("; ")
        if perms[0] == "d":
            user_w = user in users and perms[2] == "w"
            group_w = group in groups and perms[5] == "w"
            others_w = perms[8] == "w"

            if user_w or group_w or others_w:
                packages.add(name)

    if packages:
        return DatasourceProvider(
            content=sorted(packages), relative_path="insights_commands/system_user_dirs"
        )



import doctest
import pytest
from insights.tests import context_wrap
from insights.parsers import rdma_config as scc, SkipException

RDMA_CONFIG = """
# Load IPoIB
IPOIB_LOAD=yes
# Load SRP (SCSI Remote Protocol initiator support) module
SRP_LOAD=yes
# Load SRPT (SCSI Remote Protocol target support) module
SRPT_LOAD=yes
# Load iSER (iSCSI over RDMA initiator support) module
ISER_LOAD=yes
# Load iSERT (iSCSI over RDMA target support) module
ISERT_LOAD=yes
# Load RDS (Reliable Datagram Service) network protocol
RDS_LOAD=no
# Load NFSoRDMA client transport module
XPRTRDMA_LOAD=yes
# Load NFSoRDMA server transport module
SVCRDMA_LOAD=no
# Load Tech Preview device driver modules
TECH_PREVIEW_LOAD=no
# Should we modify the system mtrr registers?  We may need to do this if you
# get messages from the ib_ipath driver saying that it couldn't enable
# write combining for the PIO buffs on the card.
#
# Note: recent kernels should do this for us, but in case they don't, we'll
# leave this option
FIXUP_MTRR_REGS=no
"""

RDMA_CONFIG_INPUT_EMPTY = """
# Load IPoIB
#IPOIB_LOAD=yes
# Load SRP (SCSI Remote Protocol initiator support) module
#SRP_LOAD=yes
# Load SRPT (SCSI Remote Protocol target support) module
#SRPT_LOAD=yes
# Load iSER (iSCSI over RDMA initiator support) module
#ISER_LOAD=yes
# Load iSERT (iSCSI over RDMA target support) module
#ISERT_LOAD=yes
# Load RDS (Reliable Datagram Service) network protocol
#RDS_LOAD=no
# Load NFSoRDMA client transport module
#XPRTRDMA_LOAD=yes
# Load NFSoRDMA server transport module
#SVCRDMA_LOAD=no
# Load Tech Preview device driver modules
#TECH_PREVIEW_LOAD=no
# Should we modify the system mtrr registers?  We may need to do this if you
# get messages from the ib_ipath driver saying that it couldn't enable
# write combining for the PIO buffs on the card.
#
# Note: recent kernels should do this for us, but in case they don't, we'll
# leave this option
#FIXUP_MTRR_REGS=no
"""


def test_rdma_config():
    rdma_config = scc.RdmaConfig(context_wrap(RDMA_CONFIG))
    assert rdma_config["IPOIB_LOAD"] == 'yes'
    assert rdma_config["SRP_LOAD"] == 'yes'
    assert rdma_config["SVCRDMA_LOAD"] == 'no'


def test_rdma_config_empty():
    with pytest.raises(SkipException):
        scc.RdmaConfig(context_wrap(RDMA_CONFIG_INPUT_EMPTY))


def test_rdma_config_doc():
    env = {
            'rdma_conf': scc.RdmaConfig(context_wrap(RDMA_CONFIG)),
          }
    failed, total = doctest.testmod(scc, globs=env)
    assert failed == 0



"""
HDBVersion - Commands
=====================

Shared parser for parsing output of the ``sudo -iu <SID>adm HDB version``
commands.

"""
from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.sap_hdb_version)
class HDBVersion(CommandParser, dict):
    """
    Class for parsing the output of `HDB version` command.

    Typical output of the command is::

        # sudo -iu sr1adm HDB version
        HDB version info:
          version:             2.00.030.00.1522210459
          branch:              hanaws
          machine config:      linuxx86_64
          git hash:            bb2ff6b25b8eab5ab382c170a43dc95ae6ce298f
          git merge time:      2018-03-28 06:14:19
          weekstone:           2018.13.0
          cloud edition:       0000.00.00
          compile date:        2018-03-28 06:19:13
          compile host:        ld2221
          compile type:        rel

    Attributes:
        version (str): the raw HDB version
        major (str): the major version
        minor (str): the minor version
        revision (str): the SAP HANA SPS revision number
        patchlevel (str): the patchlevel number of this revision
        sid (str): the SID of this SAP HANA

    Examples:
        >>> type(hdb_ver)
        <class 'insights.parsers.sap_hdb_version.HDBVersion'>
        >>> hdb_ver.sid
        'sr1'
        >>> hdb_ver.version
        '2.00.030.00.1522210459'
        >>> hdb_ver.major
        '2'
        >>> hdb_ver.minor
        '00'
        >>> hdb_ver.revision
        '030'
        >>> hdb_ver.patchlevel
        '00'
        >>> hdb_ver['machine config']
        'linuxx86_64'
    """

    def parse_content(self, content):
        _ignore_bad_lines = [
            'HDB: Command not found',
            'standard error',
            'does not exist',
        ]
        if len(content) <= 1:
            raise SkipException("Incorrect content.")
        data = {}
        self.sid = self.version = self.revision = None
        self.major = self.minor = self.patchlevel = None
        # get the "sid" from the file_name: "sudo_-iu_<sid>adm_HDB_version"
        if self.file_name and 'adm' in self.file_name:
            self.sid = [i for i in self.file_name.split('_') if i.endswith('adm')][0][:-3]
        for line in content:
            # Skip unexpected lines
            if ':' not in line or any(i in line for i in _ignore_bad_lines):
                continue
            key, val = [i.strip() for i in line.split(':', 1)]
            data[key] = val
            if key == 'version':
                self.version = val
                val_splits = val.split('.')
                if len(val_splits) != 5:
                    raise SkipException("Incorrect HDB version: {0}.".format(val))
                self.major = val_splits[0]
                self.minor = val_splits[1]
                self.revision = val_splits[2]
                self.patchlevel = val_splits[3]
        if not self.version:
            raise SkipException("Incorrect content.")

        self.update(data)

    @property
    def data(self):
        return self



from insights.tests import context_wrap
from insights.parsers.httpd_log import HttpdSSLErrorLog, HttpdErrorLog
from insights.parsers.httpd_log import HttpdSSLAccessLog, HttpdAccessLog

from datetime import datetime

SSL_ACCESS_LOG = """
10.68.5.20 - - [29/Mar/2017:05:57:21 -0400] "GET / HTTP/1.1" 403 202 "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:59:38 -0400] "GET / HTTP/1.1" 200 84 "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
""".strip()


ACCESS_LOG = """
10.68.5.20 - - [29/Mar/2017:05:57:21 -0400] "GET / HTTP/1.1" 403 202 "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:58:54 -0400] "GET / HTTP/1.1" 403 202 "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:59:38 -0400] "GET / HTTP/1.1" 200 84 "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:59:41 -0400] "GET / HTTP/1.1" 304 - "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:59:43 -0400] "GET / HTTP/1.1" 304 - "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:05:59:44 -0400] "GET / HTTP/1.1" 304 - "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:06:01:13 -0400] "GET / HTTP/1.1" 304 - "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:06:01:17 -0400] "GET / HTTP/1.1" 304 - "-" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
10.68.5.20 - - [29/Mar/2017:21:47:54 -0400] "GET /favicon.ico HTTP/1.1" 404 209 "http://10.66.208.208/" "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36"
""".strip()

ERROR_LOG = """
[Tue Mar 28 03:56:00.804140 2017] [core:notice] [pid 4343:tid 139992054929536] AH00094: Command line: '/usr/sbin/httpd -D FOREGROUND'
[Tue Mar 28 03:56:38.610607 2017] [mpm_worker:notice] [pid 4343:tid 139992054929536] AH00296: caught SIGWINCH, shutting down gracefully
[Tue Mar 28 03:56:39.737815 2017] [suexec:notice] [pid 4471:tid 140592082000000] AH01232: suEXEC mechanism enabled (wrapper: /usr/sbin/suexec)
AH00558: httpd: Could not reliably determine the server's fully qualified domain name, using fe80::21a:4aff:fe01:160. Set the 'ServerName' directive globally to suppress this message
[Tue Mar 28 03:56:39.771605 2017] [auth_digest:notice] [pid 4471:tid 140592082000000] AH01757: generating secret for digest authentication ...
[Tue Mar 28 03:56:39.772272 2017] [lbmethod_heartbeat:notice] [pid 4471:tid 140592082000000] AH02282: No slotmem from mod_heartmonitor
[Tue Mar 28 03:56:39.772364 2017] [ssl:warn] [pid 4471:tid 140592082000000] AH01873: Init: Session Cache is not configured [hint: SSLSessionCache]
[Tue Mar 28 03:56:39.775833 2017] [mpm_worker:notice] [pid 4471:tid 140592082000000] AH00292: Apache/2.4.6 (Red Hat Enterprise Linux) OpenSSL/1.0.1e-fips configured -- resuming normal operations
""".strip()

HTTPD24_ERROR_LOG = """
[Fri Mar 29 01:42:23.497294 2019] [suexec:notice] [pid 1967] AH01232: suEXEC mechanism enabled (wrapper: /opt/rh/jbcs-httpd24/root/usr/sbin/suexec)
[Fri Mar 29 01:42:23.498726 2019] [:notice] [pid 1967] ModSecurity for Apache/2.9.1 (http://www.modsecurity.org/) configured.
[Fri Mar 29 01:45:23.498736 2019] [:notice] [pid 1967] ModSecurity: APR compiled version="1.6.3"; loaded version="1.6.3-31"
[Fri Mar 29 01:45:23.498743 2019] [:notice] [pid 1967] ModSecurity: PCRE compiled version="8.32 "; loaded version="8.32 2012-11-30"
[Fri Mar 29 01:45:23.498745 2019] [:notice] [pid 1967] ModSecurity: LUA compiled version="Lua 5.1"
[Fri Mar 29 01:47:23.498747 2019] [:notice] [pid 1967] ModSecurity: LIBXML compiled version="2.9.1"
[Fri Mar 29 01:47:23.498749 2019] [:notice] [pid 1967] ModSecurity: Status engine is currently disabled, enable it by set SecStatusEngine to On.
"""

JBCS_HTTPD24_ERROR_LOG = """
[Wed Apr 03 03:52:39.014686 2019] [core:notice] [pid 4499] SELinux policy enabled; httpd running as context system_u:system_r:httpd_t:s0
[Wed Apr 03 03:54:39.016900 2019] [suexec:notice] [pid 4499] AH01232: suEXEC mechanism enabled (wrapper: /opt/rh/httpd24/root/usr/sbin/suexec)
[Wed Apr 03 03:55:39.038125 2019] [http2:warn] [pid 4499] AH10034: The mpm module (prefork.c) is not supported by mod_http2. The mpm determines how things are processed in your server. HTTP/2 has more demands in this regard and the currently selected mpm will just not do. This is an advisory warning. Your server will continue to work, but the HTTP/2 protocol will be inactive.
[Wed Apr 03 03:56:39.038140 2019] [http2:warn] [pid 4499] AH02951: mod_ssl does not seem to be enabled
[Wed Apr 03 03:57:39.038835 2019] [lbmethod_heartbeat:notice] [pid 4499] AH02282: No slotmem from mod_heartmonitor
"""

SSL_ERROR_LOG = """
[Tue Mar 28 03:56:00.804140 2017] [core:notice] [pid 4343:tid 139992054929536] AH00094: Command line: '/usr/sbin/httpd -D FOREGROUND'
[Tue Mar 28 03:56:38.610607 2017] [mpm_worker:notice] [pid 4343:tid 139992054929536] AH00296: caught SIGWINCH, shutting down gracefully
[Tue Mar 28 03:56:39.737815 2017] [suexec:notice] [pid 4471:tid 140592082000000] AH01232: suEXEC mechanism enabled (wrapper: /usr/sbin/suexec)
AH00558: httpd: Could not reliably determine the server's fully qualified domain name, using fe80::21a:4aff:fe01:160. Set the 'ServerName' directive globally to suppress this message
""".strip()


def test_ssl_access_log():
    log = HttpdSSLAccessLog(context_wrap(SSL_ACCESS_LOG))
    assert 2 == len(log.get("10.68.5.20"))
    assert len(list(log.get_after(datetime(2017, 3, 29, 5, 59, 0)))) == 1


def test_access_log():
    log = HttpdAccessLog(context_wrap(ACCESS_LOG))
    assert 9 == len(log.get("10.68.5.20"))
    assert "favicon.ico" in log
    assert len(list(log.get_after(datetime(2017, 3, 29, 6, 0, 0)))) == 3


def test_ssl_error_log():
    log = HttpdSSLErrorLog(context_wrap(SSL_ERROR_LOG))
    assert 1 == len(log.get("mpm_worker:notice"))
    assert "AH00558" in log
    # Includes continuation line
    assert len(list(log.get_after(datetime(2017, 3, 28, 3, 56, 39)))) == 2


def test_error_log():
    log = HttpdErrorLog(context_wrap(ERROR_LOG))
    assert 2 == len(log.get("mpm_worker:notice"))
    assert "AH00558" in log
    # Includes continuation line
    assert len(list(log.get_after(datetime(2017, 3, 28, 3, 56, 39)))) == 6


def test_httpd24_error_log():
    log = HttpdErrorLog(context_wrap(HTTPD24_ERROR_LOG))
    assert 1 == len(log.get("suexec:notice"))
    assert "ModSecurity" in log
    # Includes continuation line
    assert len(list(log.get_after(datetime(2019, 3, 29, 1, 45, 23)))) == 5


def test_jbcs_httpd24_error_log():
    log = HttpdErrorLog(context_wrap(JBCS_HTTPD24_ERROR_LOG))
    assert 2 == len(log.get("http2:warn"))
    assert "suEXEC" in log
    # Includes continuation line
    assert len(list(log.get_after(datetime(2019, 4, 3, 3, 54, 39)))) == 4



"""
AzureInstancePlan
=================

This parser reads the output of a command
``curl -s -H Metadata:true http://169.254.169.254/metadata/instance/compute/plan?api-version=2018-10-01&format=json``,
which is used to check whether the instance is a marketplace image.

For more details, See: https://docs.microsoft.com/en-us/rest/api/compute/virtualmachines/createorupdate#plan

"""
import json

from insights.parsers import SkipException, ParseException
from insights import parser, CommandParser
from insights.specs import Specs


@parser(Specs.azure_instance_plan)
class AzureInstancePlan(CommandParser):
    """
    Class for parsing the Azure Instance Plan returned by command
    ``curl -s -H Metadata:true http://169.254.169.254/metadata/instance/compute/plan?api-version=2018-10-01&format=json``,


    Typical Output of this command is::

        {
            "name": "planName",
            "product": "planProduct",
            "publisher": "planPublisher"
        },

    Raises:
        SkipException: When content is empty or no parse-able content.

    Attributes:
        name (str): The name of the plan for the VM Instance in Azure, e.g: rhel7
        product (str): The product of the plan for the VM Instance in Azure, e.g: RHEL
        publisher (str): The publisher of the plan for the VM Instance in Azure, e.g: Red hat
        raw (str): The full JSON of the plan returned by the ``curl`` command

    Examples:
        >>> azure_plan.name == 'planName'
        True
        >>> azure_plan.product == 'planProduct'
        True
        >>> azure_plan.publisher == 'planPublisher'
        True
    """

    def parse_content(self, content):
        if not content or 'curl: ' in content[0]:
            raise SkipException()
        try:
            plan = json.loads(content[0])
        except:
            raise ParseException("Unable to parse JSON")

        self.raw = content[0]
        self.name = plan["name"] if plan["name"] != "" else None
        self.product = plan["product"] if plan["product"] != "" else None
        self.publisher = plan["publisher"] if plan["publisher"] != "" else None

    def __repr__(self):
        return "<azure_plan_name: {n}, product: {pr}, publisher: {pu}, raw: {r}".format(
            n=self.name, pr=self.product, pu=self.publisher, r=self.raw
        )



"""
Insights Core is a data collection and analysis framework that is built for
extensibility and rapid development. It includes a set of reusable components
for gathering data in myriad ways and providing a reliable object model for it.

.. code-block: python

    >>> from insights import run
    >>> from insights.parsers import installed_rpms as rpm
    >>> lower = rpm.Rpm("bash-4.4.11-1.fc26")
    >>> upper = rpm.Rpm("bash-4.4.22-1.fc26")
    >>> results = run(rpm.Installed)
    >>> rpms = results[rpm.Installed]
    >>> rpms.newest("bash")
    "0:bash-4.4.12-7.fc26"
    >>> lower <= rpms.newest("bash") < upper
    True
"""
from __future__ import print_function
import logging
import pkgutil
import os
import sys
import yaml

from collections import defaultdict
from contextlib import contextmanager

from .core import Scannable, LogFileOutput, Parser, IniConfigFile  # noqa: F401
from .core import FileListing, LegacyItemAccess, SysconfigOptions  # noqa: F401
from .core import YAMLParser, JSONParser, XMLParser, CommandParser  # noqa: F401
from .core import AttributeDict  # noqa: F401
from .core import Syslog  # noqa: F401
from .core import taglang
from .core.archives import COMPRESSION_TYPES, extract, InvalidArchive, InvalidContentType  # noqa: F401
from .core import dr  # noqa: F401
from .core.context import ClusterArchiveContext, HostContext, HostArchiveContext, SerializedArchiveContext, ExecutionContext  # noqa: F401
from .core.dr import SkipComponent  # noqa: F401
from .core.hydration import create_context, initialize_broker  # noqa: F401
from .core.plugins import combiner, fact, metadata, parser, rule  # noqa: F401
from .core.plugins import datasource, condition, incident  # noqa: F401
from .core.plugins import make_response, make_metadata, make_fingerprint  # noqa: F401
from .core.plugins import make_pass, make_fail, make_info, make_none  # noqa: F401
from .core.filters import add_filter, apply_filters, get_filters  # noqa: F401
from .formats import get_formatter
from .parsers import get_active_lines  # noqa: F401
from .util import defaults  # noqa: F401
from .formats import Formatter as FormatterClass

from .core.spec_factory import RawFileProvider, TextFileProvider

log = logging.getLogger(__name__)


package_info = dict((k, None) for k in ["RELEASE", "COMMIT", "VERSION", "NAME"])


for name in package_info:
    package_info[name] = pkgutil.get_data(__name__, name).strip().decode("utf-8")

_COLOR = "auto"


def get_nvr():
    return "{0}-{1}-{2}".format(package_info["NAME"],
                                package_info["VERSION"],
                                package_info["RELEASE"])


@contextmanager
def get_pool(parallel, prefix, kwargs):
    """
    Yields:
        a ThreadPoolExecutor if parallel is True and `concurrent.futures` exists.
        `None` otherwise.
    """
    if parallel:
        try:
            from concurrent.futures import ThreadPoolExecutor
            with ThreadPoolExecutor(thread_name_prefix=prefix, **kwargs) as pool:
                yield pool
        except ImportError:
            yield None
    else:
        yield None


RULES_STATUS = {}
"""
Mapping of dictionaries containing nvr and commitid for each rule repo included
in this instance

{"rule_repo_1": {"version": nvr(), "commit": sha1}}
"""


def add_status(name, nvr, commit=None):
    """
    Rule repositories should call this method in their package __init__ to
    register their version information.
    """
    RULES_STATUS[name] = {"version": nvr, "commit": commit}


add_status(package_info["NAME"], get_nvr(), package_info["COMMIT"])


def process_dir(broker, root, graph, context, inventory=None, parallel=False):
    ctx, broker = initialize_broker(root, context=context, broker=broker)
    log.debug("Processing %s with %s" % (root, ctx))

    if isinstance(ctx, ClusterArchiveContext):
        from .core.cluster import process_cluster
        archives = [f for f in ctx.all_files if f.endswith(COMPRESSION_TYPES)]
        return process_cluster(graph, archives, broker=broker, inventory=inventory)

    graph = dict((k, v) for k, v in graph.items() if k in dr.COMPONENTS[dr.GROUPS.single])
    if parallel:
        with get_pool(parallel, "insights-run-pool", {"max_workers": None}) as pool:
            broker = dr.run_all(graph, broker, pool)
    else:
        broker = dr.run(graph, broker=broker)
    return broker


def _run(broker, graph=None, root=None, context=None, inventory=None, parallel=False):
    """
    run is a general interface that is meant for stand-alone scripts to use
    when executing insights components.

    Args:
        broker (Broker): Optionally pass a broker to use for evaluation. One is
            created by default, but it's often useful to seed a broker with an
            initial dependency.
        graph (function or class): The component to execute. Will only execute
            the component and its dependency graph. If None, all components with
            met dependencies will execute.
        root (str): None will cause a host collection in which command and
            file specs are run. A directory or archive path will cause
            collection from the directory or archive, and only file type specs
            or those that depend on `insights.core.context.HostArchiveContext`
            will execute.
        context (obj): The execution context that's set.
        inventory (str): Path to inventory file.
        parallel (bool): Boolean as to weather to use parallel execution or not.

    Returns:
        broker: object containing the result of the evaluation.
    """
    if not root:
        context = context or HostContext
        broker[context] = context()
        graph = dict((k, v) for k, v in graph.items() if k in dr.COMPONENTS[dr.GROUPS.single])
        if parallel:
            with get_pool(parallel, "insights-run-pool", {"max_workers": None}) as pool:
                dr.run_all(graph, broker, pool)
        else:
            return dr.run(graph, broker=broker)

    if os.path.isdir(root):
        return process_dir(broker, root, graph, context, inventory=inventory, parallel=parallel)
    else:
        with extract(root) as ex:
            return process_dir(broker, ex.tmp_dir, graph, context, inventory=inventory, parallel=parallel)


def load_default_plugins():
    dr.load_components("insights.specs.default")
    dr.load_components("insights.specs.insights_archive")
    dr.load_components("insights.specs.core3_archive")
    dr.load_components("insights.specs.sos_archive")
    dr.load_components("insights.specs.jdr_archive")
    dr.load_components("insights.specs.must_gather_archive")


def load_packages(packages):
    plugins = []
    for name in packages:
        if name not in sys.modules:
            plugins.append(name)
            dr.load_components(name, continue_on_error=False)

    return plugins


def parse_plugins(p):
    plugins = []
    if p:
        for path in p.split(","):
            path = path.strip()
            if path.endswith(".py"):
                path, _ = os.path.splitext(path)
            path = path.rstrip("/").replace("/", ".")
            plugins.append(path)
    return plugins


def apply_default_enabled(config):
    """
    Configures dr and already loaded components with a default enabled
    value.
    """
    default_enabled = config.get("default_component_enabled", True)
    for k in dr.ENABLED:
        dr.ENABLED[k] = default_enabled

    enabled = defaultdict(lambda: default_enabled)
    enabled.update(dr.ENABLED)
    dr.ENABLED = enabled


def apply_configs(config):
    """
    Configures components.

    Args:
        config (dict): a dictionary with the following keys:
            default_component_enabled (bool, optional): default value for
                whether components are enable if not specifically declared in
                the config section. Defaults to True.

            configs (list): list of dictionaries with the following keys:
                name, enabled, metadata, and timeout. All keys are optional
                except name.

                name is the prefix or exact name of any loaded component. Any
                component starting with name will have the associated
                configuration applied.

                enabled is whether the matching components will execute even if
                their dependencies are met. Defaults to True.

                timeout sets the class level timeout attribute of any component
                so long as the attribute already exists.

                metadata is any dictionary that you want to attach to the
                component. The dictionary can be retrieved by the component at
                runtime.
    """
    default_enabled = config.get("default_component_enabled", True)
    delegate_keys = sorted(dr.DELEGATES, key=dr.get_name)
    for comp_cfg in config.get("configs", []):
        name = comp_cfg.get("name")
        for c in delegate_keys:
            delegate = dr.DELEGATES[c]
            cname = dr.get_name(c)
            if cname.startswith(name):
                dr.ENABLED[c] = comp_cfg.get("enabled", default_enabled)
                delegate.metadata.update(comp_cfg.get("metadata", {}))
                delegate.tags = set(comp_cfg.get("tags", delegate.tags))
                for k, v in delegate.metadata.items():
                    if hasattr(c, k):
                        log.debug("Setting %s.%s to %s", cname, k, v)
                        setattr(c, k, v)

                if hasattr(c, "timeout"):
                    c.timeout = comp_cfg.get("timeout", c.timeout)

                if hasattr(delegate, "links"):
                    delegate.links = comp_cfg.get("links", delegate.links)
            if cname == name:
                break


def _load_context(path):
    if path is None:
        return

    if "." not in path:
        path = ".".join(["insights.core.context", path])
    return dr.get_component(path)


def run(component=None, root=None, print_summary=False,
        context=None, inventory=None, print_component=None):

    args = None
    formatters = None

    if print_summary:
        import argparse
        import logging
        p = argparse.ArgumentParser(add_help=False)
        p.add_argument("archive", nargs="?", help="Archive or directory to analyze.")
        p.add_argument("-b", "--bare",
                       help='Specify "spec=filename[,spec=filename,...]" to use the bare file for the spec', default="")
        p.add_argument("-c", "--config", help="Configure components.")
        p.add_argument("-f", "--format", help="Output format.", default="insights.formats.text")
        p.add_argument("-i", "--inventory", help="Ansible inventory file for cluster analysis.")
        p.add_argument("-k", "--pkg-query", help="Expression to select rules by package.")
        p.add_argument("-p", "--plugins", default="",
                       help="Comma-separated list without spaces of package(s) or module(s) containing plugins.")
        p.add_argument("-s", "--syslog", help="Log results to syslog.", action="store_true")
        p.add_argument("-v", "--verbose", help="Verbose output.", action="store_true")
        p.add_argument("-D", "--debug", help="Verbose debug output.", action="store_true")
        p.add_argument("--color", default="auto", choices=["always", "auto", "never"], metavar="[=WHEN]",
                       help="Choose if and how the color encoding is outputted. When is 'always', 'auto', or 'never'.")
        p.add_argument("--context", help="Execution Context. Defaults to HostContext if an archive isn't passed.")
        p.add_argument("--no-load-default", help="Don't load the default plugins.", action="store_true")
        p.add_argument("--parallel", help="Execute rules in parallel.", action="store_true")
        p.add_argument("--tags", help="Expression to select rules by tag.")

        class Args(object):
            pass

        formatters = []
        args = Args()
        p.parse_known_args(namespace=args)
        p = argparse.ArgumentParser(parents=[p])

        if not args.no_load_default:
            load_default_plugins()

        global _COLOR
        _COLOR = args.color

        args.format = "insights.formats._json" if args.format == "json" else args.format
        args.format = "insights.formats._yaml" if args.format == "yaml" else args.format
        fmt = args.format if "." in args.format else "insights.formats." + args.format

        Formatter = dr.get_component(fmt)
        if not Formatter or not isinstance(Formatter, FormatterClass):
            dr.load_components(fmt, continue_on_error=False)
            Formatter = get_formatter(fmt)

        Formatter.configure(p)
        p.parse_args(namespace=args)
        formatter = Formatter(args)
        formatters.append(formatter)

        if args.syslog:
            fmt = "insights.formats._syslog"
            Formatter = dr.get_component(fmt)
            if not Formatter:
                dr.load_components(fmt, continue_on_error=False)
                Formatter = get_formatter(fmt)
            p.parse_args(namespace=args)
            formatter = Formatter(args)
            formatters.append(formatter)

        logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO if args.verbose else logging.ERROR)
        context = _load_context(args.context) or context
        inventory = args.inventory

        root = args.archive or root
        if root:
            root = os.path.realpath(root)

        plugins = parse_plugins(args.plugins)
        for p in plugins:
            dr.load_components(p, continue_on_error=False)

        if args.config:
            with open(args.config) as f:
                config = (yaml.safe_load(f))
                packages_loaded = load_packages(config.get('packages', []))
                plugins.extend(packages_loaded)
                apply_default_enabled(config)
                apply_configs(config)

        if component is None:
            component = []
            plugins = tuple(plugins)
            for c in dr.DELEGATES:
                if c.__module__.startswith(plugins):
                    component.append(c)

    if component:
        if not isinstance(component, (list, set)):
            component = [component]

        if args and args.pkg_query:
            pred = taglang.parse(args.pkg_query)
            component = [c for c in component if pred([dr.get_module_name(c)])]
            if not component:
                msg = "No components for pkg-query expression: %s" % args.pkg_query
                raise Exception(msg)

        if args and args.tags:
            pred = taglang.parse(args.tags)
            component = [c for c in component if pred(dr.get_tags(c))]
            if not component:
                msg = "No components for tag expression: %s" % args.tags
                raise Exception(msg)

        graph = {}
        for c in component:
            graph.update(dr.get_dependency_graph(c))
    else:
        graph = dr.COMPONENTS[dr.GROUPS.single]

    broker = dr.Broker()

    if args and args.bare:
        ctx = ExecutionContext()  # dummy context that no spec depend on. needed for filters to work
        specs = parse_specs(args.bare)
        specs = load_specs(specs, ctx)

        broker = dr.Broker()
        broker[ExecutionContext] = ctx
        for spec, content in specs.items():
            broker[spec] = content if dr.DELEGATES[spec].multi_output else content[-1]
    try:
        if formatters:
            for formatter in formatters:
                formatter.preprocess(broker)

            if args:
                if args.bare:
                    broker = dr.run(graph, broker=broker)
                else:
                    broker = _run(broker, graph, root, context=context, inventory=inventory, parallel=args.parallel)
            else:
                broker = _run(broker, graph, root, context=context, inventory=inventory)

            for formatter in formatters:
                formatter.postprocess(broker)
        elif print_component:
            if args:
                if args.bare:
                    broker = dr.run(graph, broker=broker)
                else:
                    broker = _run(broker, graph, root, context=context, inventory=inventory, parallel=args.parallel)
            else:
                broker = _run(broker, graph, root, context=context, inventory=inventory)

            broker.print_component(print_component)
        else:
            if args:
                if args.bare:
                    broker = dr.run(graph, broker=broker)
                else:
                    broker = _run(broker, graph, root, context=context, inventory=inventory, parallel=args.parallel)
            else:
                broker = _run(broker, graph, root, context=context, inventory=inventory)

        return broker
    except (InvalidContentType, InvalidArchive):
        if args and args.archive:
            path = args.archive
            msg = "Invalid directory or archive. Did you mean to pass -p {p}?"
            log.error(msg.format(p=path))
        else:
            raise


def parse_specs(specs):
    """
    -b "hostname=/etc/hostname, redhat_release=/etc/redhat-release, .."
    """
    return dict(s.strip().split("=", 1) for s in specs.split(","))


def load_datasource(name):
    name = "insights.specs.Specs." + name if "." not in name else name
    return dr.get_component(name)


def load_specs(specs, ctx):
    results = defaultdict(list)
    for spec, path in specs.items():
        s = load_datasource(spec)
        root = "/" if path.startswith('/') else "."
        if s.raw:
            c = RawFileProvider(relative_path=path, root=root, ds=s, ctx=ctx)
        else:
            c = TextFileProvider(relative_path=path, root=root, ds=s, ctx=ctx)
        results[s].append(c)
    return results


def main():
    if "" not in sys.path:
        sys.path.insert(0, "")
    run(print_summary=True)


if __name__ == "__main__":
    main()
